// VinciaCommon.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2011 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the auxiliary helper 
// functions for VINCIA, including the VinciaCommon and AlphaQCD classes, 
// the latter of which inherits from Pythia8::AlphaStrong, but adds a bit 
// of extra functionality.

#ifndef VINCIACOMMON_H
#define VINCIACOMMON_H

// Include Pythia 8 headers and name space

#include "Basics.h"
#include "Event.h"
#include "PythiaStdlib.h"

using namespace Pythia8;

// ------------ GLOBAL VINCIA CONSTANTS AND FUNCTIONS --------------------
namespace Vincia {

  // Global numerical precision target
  // (should be kept within a reasonable margin of machine precision)
  // (large values may result in strange results due to non-zero terms
  //  being artificially forced to zero)
  const double TINY    = 1.0e-9;
  
  //**************************************************************************

  // Utilities for printing out VINCIA Messages
  void printErr(string,string);
  void printOut(string,string);
   
  // Utilities for converting data to strings
  string num2str(int,int width=7) ;
  string num2str(double,int width=9) ;
  string bool2str(bool) ;
  
  //**************************************************************************

  // VinciaCommon: 
  // Contains functions and variables shared by VinciaShower and VinciaMatching

  class VinciaCommon {
    
  public:
    
    // Constructor.
    VinciaCommon() {isInitPtr=false; isInit=false;}

    // Destructor.
    virtual ~VinciaCommon() {}

    // 
    bool initPtr(ParticleData* particleDataPtrIn, Settings* settingsPtrIn) {
      particleDataPtr = particleDataPtrIn; 
      settingsPtr = settingsPtrIn; 
      isInitPtr=true;
      return true;
    }

    // 
    bool init() {
      // Check initPtr
      if (!isInitPtr) {
	if (verbose >= 1) 
	  printOut("VinciaCommon::init","Error! pointers not initialized"); 
	return false;
      }
      // Quark and lepton mass flags
      isMassiveS         = settingsPtr->flag("VinciaShower:isMassiveS");
      isMassiveC         = settingsPtr->flag("VinciaShower:isMassiveC");
      isMassiveB         = settingsPtr->flag("VinciaShower:isMassiveB");
      isMassiveTau       = settingsPtr->flag("VinciaShower:isMassiveTau");
      ms                 = particleDataPtr->m0(3);
      mc                 = particleDataPtr->m0(4);
      mb                 = particleDataPtr->m0(5);
      verbose            = settingsPtr->mode("Vincia:verbose");
      // Return
      isInit = true;
      return true;
    };

    // Function to determine if a particle is to be treated as massive
    bool isMassive(int id) {
      if (!isInit) {
	if (verbose >= 1) 
	  printOut("VinciaCommon::isMassive","Error! not initialized"); 
	return false;
      }
      // Immediately return false for gluons and photons
      if (id == 21 || id == 22) return false;
      // Otherwise check id and mass against flags
      int ida   = abs(id);
      double m0 = particleDataPtr->m0(ida);
      // MR 14.3.2011: return false if m0 < TINY, 
      // disregarding isMassiveX
      if (m0 < TINY) return false;
      else if (isMassiveS && (m0 >= ms || ida == 3)) return true;
      else if (isMassiveC && (m0 >= mc || ida == 4)) return true;
      else if (isMassiveB && (m0 >= mb || ida == 5)) return true;
      else if (isMassiveTau && ida == 15) return true;
      else return false;
    }

    // Verbosity level
    void setVerbose(int verboseIn) { verbose = verboseIn;}

  private:

    // Flags for which particles are massive
    bool isMassiveS, isMassiveC, isMassiveB, isMassiveTau;
    double ms, mc, mb;

    // Pointers
    Settings*      settingsPtr; 
    ParticleData*  particleDataPtr;

    // Internal flags and settings
    bool isInitPtr, isInit;
    int verbose;

  };
 
  //**************************************************************************
  
  // The AlphaQCD class inherits from Pythia8::AlphaStrong, but can be 
  // expanded to include additional functionality without affecting 
  // Pythia 8's internally used couplings. 
  
  class AlphaQCD : public Pythia8::AlphaStrong {
    
  public: 
    
    // Constructors.
    AlphaQCD() { isInit=false; }
    AlphaQCD(double valueIn, int orderIn = 1, int verbose = 1) { 
      if (valueIn <= 0.0) printOut("AlphaQCD","Error. Attempting to initialize with aS(mZ) = "+num2str(valueIn)+" order = "+num2str(orderIn));
      else if (verbose >= 9) {
	printOut("AlphaQCD","initializing with aS(mZ) = "
				 +num2str(valueIn)+" order = "+num2str(orderIn));
	init( valueIn, orderIn) ; 
      }
    }
    
    // ---- ORDINARY FUNCTIONALITY INHERITED FROM Pythia8::AlphaStrong ----
    // ... 
    
    // ---- Additional Functionality ----
    
    // alpha explicitly of Q and Q2, respectively
    double alphaSofQ(double Q) {
      return alphaS(pow2(Q));
    }
    double alphaSofQ2(double Q2) {
      return alphaS(Q2);
    }
    
  protected: 
    
  private:

  }; // End class AlphaQCD
  
  //**************************************************************************
  
  // Globally accessible functions and constants in the Vincia namespace
  
  // Color factors in Vincia normalization
  const double CA=3.0;
  const double CF=8.0/3.0;
  const double TR=1.0;
  const double NC=3.0;

  // Indices of antenna functions
  const int iqqEmit  = 0;
  const int iqgEmit  = 1;
  const int iqgSplit = 2;
  const int iggEmit  = 3;
  const int iggSplit = 4;

  // Mathematical constants
  const double gammaE=0.577215664901532860606512090082402431042;
  
  // Max index for antenna coefficients
  const int LaurentIndexMax  = 10;
  // MR 10.7.2010: Max index for the invariants in the massive
  // antennae
  const int LaurentIndexMaxMassiveInv = 4;
  const int LaurentIndexMaxMassiveMass = 8;
  
  // A few useful auxiliary functions
  double m(const Vec4&);
  double m2(const Vec4&);
  double m2(const Vec4&, const Vec4&, const Vec4&);
  double m2(const Vec4&, const Vec4&, const Vec4&, const Vec4&);
  double m2(const Particle&, const Particle&, const Particle&);
  void Vec4toD4(const Vec4&, double*);
  void Vec4toMG(const Vec4&, double*);
  
  // Utilities to find and sort a bunch of partons
  int colorSort(Event&, vector<int>&);
  int findDaughters(Event&, int, vector<int>&);
  // Function to return whether parton i in event has iAncestor in its history
  bool isAncestor(Event& event, int i, int iAncestor);
  
  // Functors seem to be the only way to use zbrent for the rescaling 
  // variable needed in the massive Rambo version
  // TFunctor should be a template class, but the generality is 
  // not needed at this point
  
  // pure virtual function overloading operator()
  class TFunctor {
    public:
      virtual double operator()(double) =0;
  };
  
  class TXiFunctor : public TFunctor {
    public:
      // constructor takes a vector with the masses and one with the 
      // energies
      TXiFunctor(vector<double> mIn, vector<double> energiesIn);
      double operator() (double); 
    private:
      vector<double> m;
      vector<double> energies;
  };
  
  // derive a class which can wrap function pointers
  class TPtrFunctor : public TFunctor {
    public:
      TPtrFunctor(double (*fPtrIn)(double) ){ fPtr = fPtrIn; };
      double operator() (double arg) { return fPtr(arg); };
    private:
      // pointer to a function taking one double and returning one
      // double
      double (*fPtr)(double);
  };

  
  // Math support tools
  double zbrent(double (*)(double), double, double, double, double);
  // zbrentAlt is code duplication of zbrent, should replace zbrent, 
  // but testing is not complete
  // at present, doesn't work if the reference to TFunctor is declared
  // const
  double zbrentAlt(TFunctor&, double, double, double, double);
  double integrateGauss( double (*)(double), double, double, double);
  void makeSpline(vector<double>&, vector<double>&, vector<double>&);
  double getSpline(const double, vector<double>&, vector<double>&, 
		   vector<double>&);
  
  // Math support functions
  double Li2(const double,const double kmax=100.0,const double xerr=TINY);
  double lnGamma(const double);
  double Gamma(const double);
  int factorial(const int);
  int binomial(const int,int);
  
  // MR 21.12.2010: Gram determinant
  // the inviariants used in the argument are 2*pi*pj
  double gDet(double s12tilde, double s23tilde, double s13tilde, 
	      double m1, double m2, double m3);
  double gDet(Vec4 p1, Vec4 p2, Vec4 p3);
  
  // MR 21.12.2010: Minimal value of y23tilde = 2*p2*p3/(p1+p2+p3)^2 
  // as a function of y12tilde = 2*p1*p2/(p1+p2+p3)^2 and the
  // reduced masses m_i/sqrt((p1+p2+p3)^2)
  double y23tildeMin(double y12tilde, double mu1, double mu2, double mu3);
  // the same for the maximum value of y23tilde
  double y23tildeMax(double y12tilde, double mu1, double mu2, double mu3);  
  
  //**************************************************************************
  
  // A simple class to contain a spline
  
  class spline {
  public: 
    
    // constructor
    spline() {};
    
    // Members
    vector<double> x, y, d2;
  };
  
  //**************************************************************************

      
} // End VINCIA namespace
#endif
