// VinciaShower.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the VinciaShower class 
  
#ifndef VINCIA_VinciaShower_H
#define VINCIA_VinciaShower_H

#include "TimeShower.h"
#include "VinciaAntenna.h"
#include "VinciaCommon.h"
#include "VinciaEvolutionVariables.h"
#include "VinciaMatching.h"

using namespace Pythia8;

// Declare the F77 3->2 clustering map for use below
extern"C" {
  void vc3to2_(int *MMAP, double PA[4], double PR[4], double PB[4],
	       double PAHAT[4], double PBHAT[4], int *IFAIL);
}

namespace Vincia {
  
  // Classes contained in this file (forward declarations)
  class Branchelemental;
  class VinciaShower;  
  
//**************************************************************************

// The VinciaShower class: main shower class for the timelike antenna shower   
// Inherits from TimeShower in Pythia 8 so can be used as alternative to 
// TimeShower.
  
  class VinciaShower : public Pythia8::TimeShower {
    
    // Allow VinciaPlugin to access private information
    friend class VinciaPlugin;
    
  public:
   
    // ----------------------------------------------------------------------
    
    // CONSTRUCTOR AND DESTRUCTOR 
    
    // Constructor.
    VinciaShower() {};
  
    // Destructor.
    virtual ~VinciaShower() {};
  
    // -----------------------------------------------------------------------
    
    // FUNCTIONS INHERITED FROM PYTHIA 8 FOR INTERFACING 
    
    // Initialize alphaStrong and related pTmin parameters.
    virtual void init( BeamParticle* beamAPtrIn = 0, 
		       BeamParticle* beamBPtrIn = 0);
    
    // Top-level routine to do a full time-like shower in resonance decay.
    virtual int shower( int iBeg, int iEnd, Event& event, double pTmax);
    
    // Prepare system for evolution.
    virtual void prepare( int iSys, Event& event, bool limitPTmaxIn);
    // Prepare system for evolution: old format 
    // (to be backwards compatible with 8.135)
    virtual void prepare( int iSys, Event& event) {prepare(iSys, event, true);}
    
    // Update dipole list after each ISR emission.  
    virtual void update( int iSys, Event& event);
    
    // Select next pT in downwards evolution.
    virtual double pTnext( Event& event, double pTbegAll, double pTendAll);
    
    // Branch event
    virtual bool branch( Event& event, bool isInterleaved=false); 
    
    // Update dipole list after a multiple interactions rescattering.
    virtual void rescatterUpdate( int iSys, Event& event);
    
    // Propagate recoil from branching to internal lines between systems. 
    virtual bool rescatterPropagateRecoil( Event& event, Vec4& pNew);
    
    // END OF FUNCTIONS INHERITED FROM PYTHIA 8 FOR INTERFACING              
    
    // -----------------------------------------------------------------------
    
    // PUBLIC NON-INHERITED MEMBERS
    
    // Initialize Pointers to Antenna sets
    void initAntPtr(AntennaSet* antV0, AntennaSet* antV1=0, 
		    AntennaSet* antV2=0, AntennaSet* antV3=0) {
      antennaSets.resize(0);
      antennaSets.push_back(antV0);
      if (antV1 != 0) antennaSets.push_back(antV1);
      if (antV2 != 0) antennaSets.push_back(antV2);
      if (antV3 != 0) antennaSets.push_back(antV3);
    }
    
    // Initialize internal and diagnostic histograms
    void initVinciaHistos();
    
    // Utility to print dipole list; for debug mainly.
    void list( ostream& os = cout );
    
    // Print header information (version, settings, parameters, etc)
    void header();
    
    // Print list of winning branchings so far; for debug mainly.
    //virtual void history( ostream& os = cout ) {};
    
    // Print final statistics information
    void printInfo( ostream& os = cout );
    
    // Print internal and diagnostic histrograms
    void printHistos( ostream& os = cout ) {
      map<string,Hist>::iterator iH;
      for (iH=vinciaHistos.begin();iH!=vinciaHistos.end();++iH) {	
	string Hname=iH->first;
        if ( vinciaHistos[Hname].getEntries() >= 1) {
	  os<<Hname<<vinciaHistos[Hname]<<endl;
	}
      }
    }
    
    // alpha_S:
    AlphaQCD alphaS;        

    // Phase space region boundaries
    double getQMin(double Q);
    double getQMin(int Nf);
    double getQMax(int Nf);
    int    getNf(double Q);
    
   // The hadronization cutoff boundary
    bool perturbativeTheta(double, double, double);
    bool perturbativeTheta(Vec4, Vec4, Vec4);
    bool perturbativeTheta(Branchelemental*);   
    
    // Get weight of current event; 
    // Normally 1 for set 0 (user settings) 
    // Use iVar > 0 to access uncertainty weights
    double weight(int iVar=0) const {
      if (iVar >= 0 && iVar < nVariations()) 
	return weightSave[iVar] ;
      else return 0.0 ;
    }   

    // Return number of weights available
    int nVariations() const { return weightSave.size() ;}  

    // Warning flags: set true if w < 0 and/or |w| > 1 occurred, respectively 
    bool negativeWeightOccurred;
    bool nonunityWeightOccurred;
    
    // Return specific antenna (default is to take it from default antenna set)
    Antenna* getAnt(int iAnt, int iSet=0) {
      return antennaSets[iSet]->getAnt(iAnt);
    }

    // Return a2() for specific antennae, with default or non-default charge
    double a2(int iAnt,vector<Vec4> p, int a,int r,int b,double charge);
    double a2(int iAnt,vector<Vec4> p, int a,int r,int b);
    
    // Set verbose level
    void setVerbose(int verboseIn) {verbose = verboseIn;}

  private:
    
    // -----------------------------------------------------------------------
    
    // PRIVATE MEMBERS
    
    // Generate new trial scale
    double generateQnew(Branchelemental* trial, double Qstart = -1.0);

    // Main trial accept function
    bool acceptTrial(Event& event, Branchelemental* trial);

    //--------------------------------------------------------------------
    // AUXILIARIES to acceptBranching():

    // Generate new branching (+ kine checks)
    bool generateBranching(Branchelemental* trial, double QnewIn = -1.0);

    // Veto function for sector ordering.
    bool sectorAccept(Event& event, Branchelemental* trial);
    
    // Main matching routine, returns bool = accept (true) / reject (false)
    bool matchingAccept(Event& event, Branchelemental* trial);
    //--------------------------------------------------------------------
    
    // Check if particular weight exists
    bool existsWeight(int iVar=0) {
      if (iVar >= 0 && iVar < nVariations()) return true;
      else return false;
    }

    // Set all event weights (e.g., for initialization)
    void setWeightAll( double weightIn ) {
      for (int iVar=0; iVar < nVariations(); iVar++) {
	setWeight(weightIn,iVar);      
      }
    }
    
    // Set all weight sums (e.g., for initialization)
    void resetWeightSums(double weightSumIn = 0.0) {
      for (int iVar=0; iVar < nVariations(); iVar++) {
	weightSumSave[iVar]=weightSumIn;
      }
    }
    
    // Set event weight, check for negative and non-unity weights.
    bool setWeight( double weightIn, int iVarIn=0);
    
    // Scale event weight
    bool scaleWeight( double scaleFacIn, int iVarIn=0) {
      if (existsWeight(iVarIn)) {
	weightSave[iVarIn] *= scaleFacIn;	
	return true;
      } else return false;
    }

    // AlphaS parameters 
    int alphaSorder, alphaSmode; 
    double alphaSvalue, alphaSscaleFactor, alphaSMax, alphaSMuMin;
    double mt, mtb, ms, mb, mc;
    double Lambda3flav, Lambda4flav, Lambda5flav;
    double Lambda3flav2, Lambda4flav2, Lambda5flav2;

    // Shower parameters
    bool useScale;
    int evolutionType, cutoffType, cutoffMode, evolutionMode, nGluonToQuark;
    double cutoffScale, QMinAll;

    // Properties of current physical antenna
    double chargeFactor, antPhys;
    int kineMapType;

    // Uncertainty Bands
    bool uncertaintyBands;
    double uncertaintyBandsKAlphaS;

    // Pointer to Evolution Variable
    EvolutionVariable* RPtr;

    // Pointer to VINCIA common function class
    VinciaCommon* vinComPtr;

    // Sector 
    int sectorType;
    bool sectorOrdering;

    // Hyperjet on/off
    bool hyperjet;
    
    // Pointer to VINCIA clustering map
    VinClu* vinCluPtr;
    
    // Desired matching multiplicities, level, regulator, regulator scales
    Matching* matchingPtr;
    bool matching, isResonance;
    int  matchingOrder;
    int  matchingRegOrder, matchingRegShape, matchingScaleType;
    double matchingScale, matchingScaleRatio;
    bool  matchingScaleIsAbs, matchingFullColor;
    vector<double> PME;

    // HeadRoom factors to multiply trial kernels with
    vector<double> headRoomFactorEmit, headRoomFactorSplit;

    // Bool to tell whether MadGraph linked or not.
    bool isMGOn;    // True if real MG interface linked, false if dummy
    int nEmit, nSplit;   // Count orders of splittings, for matching
    int MEClass;         // Code telling which originating resonance / process
    // int nEmitPhoton, nSplitPhoton, nEmitZ, nSplitZ, nEmitW, nSplitW;
    
    // NLL Matching (still experimental)
    int showerNLL;

    // NLC corrections in showering and matching
    int showerNLC, showerNLCType;

    // Pointers to the two incoming beams.
    BeamParticle* beamAPtr;
    BeamParticle* beamBPtr;
    
    // Vector of antenna sets (0 is physical, >= 1 are variations)
    vector<AntennaSet*> antennaSets;    
    vector<double> Paccept;

    // vector of dipoles (with trial branchings) 
    vector<Branchelemental> branchelementals;

    // current winner
    Branchelemental* winner;
    // Store history of winners (e.g., to check alternative orderings)
    vector<Branchelemental> winners;

    // current global restart scale for unordered antennae
    double Qrestart;

    // current global ordering scale for unordered antennae
    double Q2UnOrdOld, mD2UnOrdOld, pT2UnOrdOld;

    // Internal histograms
    map<string,Hist> vinciaHistos;
    
    // Total mass of showering system (used for Type 3 ordering)
    double mSystem;
    
    // Event weights
    vector<double> weightSave, weightSumSave, weightSum2Save;
    vector<double> weightOld, weightMaxSave;    
    double nUnityWeight, nNonunityWeight, nNegativeWeight;
    double nTotWeights, nWeights;
    double reweightMaxPSaveP, reweightMaxPSaveQ;  
    double reweightMaxQSaveP, reweightMaxQSaveQ;
    
    // Verbose setting and version number
    int verbose;
    double version;

  };
  
  //**************************************************************************
  
  // Auxiliary classes used by VinciaShower
  
  //**************************************************************************
  

  //**************************************************************************
  
  // The Branchelemental class: contains a generic 2->3 branching, with 
  // pointers to the relevant places in the event record and a few useful 
  // functions. 
  // The member function list() prints a brief record of the contents.
  // The member function setTrial(bool) is used to (re)set whether this
  // elemental already contains a saved trial branching, or whether a new 
  // one should be generated.
  
  class Branchelemental {
    
  public:
    
    // Constructors
    // Create empty branchelemental
  Branchelemental() : system(0), unOrdering(false) {setTrial(false);};
    // Create branchelemental with iOld1In,iOld2In as parents
    // and initialize the PDT pointers for the new ones
  Branchelemental(Event& event, int iOld1In, int iOld2In, ParticleData* PDTPtrIn) 
    : system(0) {reset(event, iOld1In, iOld2In, PDTPtrIn);};
    
    // CM energy squared
    double s() {return m2(old1,old2);}
    double sNew() {return m2(new1,new2,new3);}
    // Dot products
    double s12() {return m2(new1,new2) - new1.m2() - new2.m2();} 
    double s23() {return m2(new2,new3) - new2.m2() - new3.m2();} 
    double s13() {return m2(new1,new3) - new1.m2() - new3.m2();} 
    // Mass (negative if spacelike virtual)
    double m() {return (s() > 0) ? sqrt(s()) : -sqrt(-s());}
    
    // Reset and set trial true/false
    void reset(Event& event, int iOld1In, int iOld2In, ParticleData* PDTPtrIn=0) {
      iOld1       = iOld1In;
      iOld2       = iOld2In;
      old1        = event[iOld1In];
      old2        = event[iOld2In];
      nFailed     = 0;
      Qold        = 0.0;
      iL          = 0; 
      iR          = 0;
      iLL         = 0;
      iRR         = 0;
      setTrial(false);      
      unOrdering  = false;
      branchFlavor  =-1;      
      Qnew        = 0.0;
      nFailed     = 0;
      new1=Particle(0,51,iOld1,iOld2,0,0,0,0,0.);
      new2=Particle(0,51,iOld1,iOld2,0,0,0,0,0.);
      new3=Particle(0,51,iOld2,iOld2,0,0,0,0,0.);      
      setPDTPtr(PDTPtrIn);
      headRoomFactorEmit=1.0;
      headRoomFactorSplit=1.0;
    }
    void setTrial(bool hasTrialIn) { hasTrial=hasTrialIn;}
    
    // Set particle data table pointers
    void setPDTPtr(ParticleData* particleDataPtrIn) {
      new1.setPDTPtr(particleDataPtrIn);
      new2.setPDTPtr(particleDataPtrIn);
      new3.setPDTPtr(particleDataPtrIn);
    }

    // HeadRoomFactor for current branching
    double getHeadRoomFactorEmit() {return headRoomFactorEmit;}
    void setHeadRoomFactorEmit(double headRoomFactorIn) {
      headRoomFactorEmit=headRoomFactorIn;
    }
    double getHeadRoomFactorSplit() {return headRoomFactorSplit;}
    void setHeadRoomFactorSplit(double headRoomFactorIn) {
      headRoomFactorSplit=headRoomFactorIn;
    }

    // Data storage members 
    int iOld1, iOld2;    
    Particle old1, old2;
    Particle new1, new2, new3;  
    // Nearest-Neighbour and Next-to-Nearest Neighbour indices
    int iL,iR;
    int iLL,iRR;
    // 
    int system,nFailed;
    double Qold;
    // Ultimate ancestor (e.g., if part of decay system)
    int iAncestor;
    // branchFlavor = 0: gluon emission, branchFlavor>0: gluon splitting
    int    branchFlavor;
    double Qnew;
    int    iAnt;
    // Has trial branching, and whether strong ordering should be imposed
    bool hasTrial, unOrdering;  

    // List function
    void list(ostream& os=cout, bool header=false, bool footer=false); 
    
  private:

    // HeadRoom Factors
    double headRoomFactorEmit, headRoomFactorSplit;

  };
  
}

#endif
