// VinciaShower.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// VINCIA is licenced under the GNU GPL version 2.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Function definitions (not found in the header) for the VinciaShower class, 
// as follows:
// 1) inherited member functions from the Pythia 8 TimeShower classs
// 2) non-inherited member functions for the VinciaShower class
// 3) member functions for the Branchelemental class

// Include the Vincia headers 
#include "VinciaShower.h"

using namespace Pythia8;

namespace Vincia {
  
  //**********************************************************************
  
  // The VinciaShower class and its members
  
  //**********************************************************************
  
  // BEGIN INHERITED FUNCTIONS FOR INTERFACE WITH PYTHIA8
  //  : init     : global shower initialization function
  //  : shower   : perform shower off partonic system
  //  : prepare  : prepare partonic system for showering
  //  : update   : purely dummy
  //  : rescatterUpdate : purely dummy
  //  : rescatterPropagateRecoil : purely dummy
  //  : pTnext   : generate next scale in downwards evolution
  //  : branch   : perform a branching, store results in event record
  
  //*********
  
  // Initialize Shower
  
  void VinciaShower::init( BeamParticle* beamAPtrIn, 
			   BeamParticle* beamBPtrIn) {
    
    // Verbose level and version number
    verbose            = settingsPtr->mode("Vincia:verbose");
    version            = settingsPtr->parm("Vincia:versionNumber");

    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::init","begin --------------");
    }
    
    // Store input pointers for future use. 
    beamAPtr     = beamAPtrIn;
    beamBPtr     = beamBPtrIn;
    
    // Default evolution parameters
    evolutionType      = settingsPtr->mode("VinciaShower:evolutionType");
    // Decide whether to force strong (evolution) ordering or not
    evolutionMode      = settingsPtr->mode("VinciaShower:evolutionMode");
    // For pT-evolution, ordering mode 3 == ordering mode 2
    if (evolutionType == 1 && evolutionMode == 3) evolutionMode = 2;
    // Number of active quark flavours
    nGluonToQuark      = settingsPtr->mode("VinciaShower:nGluonToQuark");

    // Decide whether to impose sector ordering or not
    sectorOrdering     = settingsPtr->flag("VinciaShower:sectorOrdering");
    
    // Flag whether to use scale() associated with particles or not
    useScale               = settingsPtr->flag("VinciaShower:useCreationScales");    
    // Enhancement factor on scale choice
    pTmaxFudge  = settingsPtr->parm("VinciaShower:pTmaxFudge");
    
    // Perturbative cutoff
    cutoffType         = settingsPtr->mode("Vincia:cutoffType");
    if (cutoffType == -1) {
      printOut("VinciaShower::init",
            "Note - using Pythia to set hadronization cutoff");    
      cutoffScale = 2.0 * settingsPtr->parm("TimeShower:pTmin");
      settingsPtr->parm("Vincia:cutoffScale",cutoffScale);
    } else {
      if (cutoffType == 0) cutoffType = evolutionType;
      if (evolutionType == 3 && cutoffType != evolutionType) {
	printOut("VinciaShower::init",
            "Warning - forced cutoffType = 1 to regulate energy ordering");
	cutoffType = 1;
      }
      cutoffScale        = settingsPtr->parm("Vincia:cutoffScale");
    }    

    // Uncertainty Bands
    uncertaintyBands        = settingsPtr->flag("Vincia:uncertaintyBands");
    uncertaintyBandsKAlphaS = settingsPtr->parm("Vincia:uncertaintyBandsKAlphaS");

    // Uncertainty Bands not yet implemented for sector showers
    if (sectorOrdering && uncertaintyBands) {
      uncertaintyBands = false;
      settingsPtr->flag("Vincia:uncertaintyBands",false);
      if (verbose >= 2) printOut("VinciaShower::init",
	"uncertainty bands are switched off for sector showers.");
    }

    // Number of uncertainty variations (including USR)
    // Hardcoded here, no sense to make user specifiable, since user 
    // can see a table of which variations are being performed, and can
    // then just choose the weight sets he or she wants to consider. 
    nWeights = 1;
    if (uncertaintyBands) nWeights = 12;
    // Resize weight and Paccept arrays to the relevant number of elements
    weightSave.resize(nWeights);
    weightSumSave.resize(nWeights);
    weightSum2Save.resize(nWeights);
    weightOld.resize(nWeights);
    weightMaxSave.resize(nWeights);
    for (int iVar=0; iVar<nWeights; iVar++) weightMaxSave[iVar]=0.0;
    Paccept.resize(nWeights);

    // Hyperjet compatibility mode on/off
    hyperjet           = settingsPtr->flag("Vincia:hyperjet");    

    // Initialize Evolution Variable
    RPtr->init();

    // ME Matching (default: matching = on and strategy = multiplicative)
    matching           = true;

    // ME Matching not yet implemented for sector showers
    if (sectorOrdering && matching) {
      matching = false;
      if (verbose >= 1) printOut("VinciaShower::init",
	"matching is switched off for sector showers.");
    }

    // Matching settings
    matchingOrder       = settingsPtr->mode("VinciaMatching:order");
    matching            = matching && matchingOrder >= 1;
    matchingRegOrder    = settingsPtr->mode("VinciaMatching:regOrder");
    matchingRegShape    = settingsPtr->mode("VinciaMatching:regShape"); 
    matchingScaleType   = settingsPtr->mode("VinciaMatching:regType");
    matchingScaleIsAbs  = settingsPtr->flag("VinciaMatching:regScaleIsAbsolute");
    matchingScale       = settingsPtr->parm("VinciaMatching:regScale");
    matchingScaleRatio  = settingsPtr->parm("VinciaMatching:regScaleRatio");

    // Initialize headroom factors for all matched orders
    double headRoomFactor      = settingsPtr->parm("Vincia:headRoomFactor");
    if (! matching) headRoomFactor = 1.0;
    headRoomFactorEmit.resize(matchingOrder+1);
    headRoomFactorSplit.resize(matchingOrder+1);
    for (int order=0; order<=matchingOrder; order++) {
      headRoomFactorEmit[order]=headRoomFactor;
      headRoomFactorSplit[order]=headRoomFactor;
    }

    // Internal Histograms
    if (verbose >= 2) initVinciaHistos();
    
    // alphaS
    alphaSvalue        = settingsPtr->parm("Vincia:alphaSvalue");
    alphaSorder        = settingsPtr->mode("Vincia:alphaSorder");
    alphaSscaleFactor  = settingsPtr->parm("Vincia:alphaSscaleFactor");
    alphaSmode         = settingsPtr->mode("Vincia:alphaSmode"); 
    alphaSMax          = settingsPtr->parm("Vincia:alphaSMax"); 

    // If forcing pT as renormalization scale, but pT used as evolution scale,
    // revert to QE as scale (increases efficiency)
    if (alphaSmode == 3 && evolutionType == 1) alphaSmode = 1;

    // Initialize alphaS
    alphaS.init(alphaSvalue, alphaSorder); 

    // Check freezeout scale
    alphaSMuMin = settingsPtr->parm("Vincia:alphaSMuMin"); 
    if (alphaSorder >= 1) {
      // Make sure smallest ren scale > Lambda3
      alphaSMuMin = max(alphaSMuMin,1.1*alphaS.Lambda3()); 
    }
    alphaSMax = min(alphaSMax,alphaS.alphaS(pow2(alphaSMuMin)));

    // Lambda for 5, 4 and 3 flavours.
    Lambda3flav        = alphaS.Lambda3(); 
    Lambda4flav        = alphaS.Lambda4(); 
    Lambda5flav        = alphaS.Lambda5(); 
    Lambda5flav2       = pow2(Lambda5flav);
    Lambda4flav2       = pow2(Lambda4flav);
    Lambda3flav2       = pow2(Lambda3flav);
     
    // Set absolute lower limit on evolution
    if (alphaSorder == 1) {
      QMinAll = 1.1*Lambda3flav/alphaSscaleFactor;      
    } else {
      QMinAll = 0.1; 
    }
    if (evolutionType == cutoffType || cutoffType == 0) {
      QMinAll     = max(QMinAll,cutoffScale);
    } 
    cutoffScale = max(cutoffScale,QMinAll);

    // Masses
    // MR 3.1.2011: use m0() (mass() returns according to a Breit-Wigner)
    mt          = particleDataPtr->m0(6);
    mb          = particleDataPtr->m0(5);
    mtb         = sqrt(mt*mb);
    mc          = particleDataPtr->m0(4);
    ms          = particleDataPtr->m0(3);
    
    // NLL Shower 
    showerNLL = settingsPtr->mode("VinciaShower:NLL");
    if (showerNLL >= 1) {
      // Switch off NLL if using constant alphaS
      if (alphaSorder <= 0) {
	if (verbose >= 1) printOut("VinciaShower::init",
  	   "Cannot do NLL with constant alphaS. NLL matching switched off.");
	showerNLL = 0;
      }
    }

    // NLC Shower
    showerNLC         = settingsPtr->mode("VinciaShower:NLC");

    // Matching at full or leading color
    matchingFullColor =  settingsPtr->flag("VinciaMatching:fullColor");

    // Warning if inconsistent
    if ( showerNLC >= 1 && !matchingFullColor ) {
      if (verbose >= 1) printOut("VinciaShower::init",
	    "Warning: matching to LC but showering to NLC is inconsistent");
    }

    // MR 5.11.2010: initialise 
    // nNegativeWeight, nNonunityWeight and nTotWeights to 0
    nNegativeWeight		   = 0;
    nNonunityWeight		   = 0;
    nTotWeights			   = 0;
 
    // Initialize negative-weight and non-unity warning flags    
    negativeWeightOccurred = false;
    nonunityWeightOccurred = false;
    setWeightAll(0.0);
    resetWeightSums(0.0);
    reweightMaxPSaveP      = 0.0;
    reweightMaxPSaveQ      = 0.0;
    reweightMaxQSaveP      = 0.0;
    reweightMaxQSaveQ      = 0.0;
    
    // Initialize VinClu, VinCom, and MadGraph interface objects,
    vinCluPtr->init();
    vinComPtr->init();
    // Check MG interface and initialize matching class
    isMGOn = settingsPtr->flag("Vincia:MGInterface");
    if (isMGOn) {
      isMGOn = matchingPtr->init(nWeights);
      settingsPtr->flag("Vincia:MGInterface",isMGOn);
      if (isMGOn && verbose >= 2) {
	printOut("VinciaPlugin()","MGInterface switched on.");
      }
    }

    // Print VINCIA header and list of parameters
    if (verbose >= 1) header();    
    
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::init","end --------------");
    }
    
  }
  
  //*********
  
  // Top-level routine to do a full time-like shower in resonance decay.
  
  int VinciaShower::shower(int iBeg, int iEnd, Event& event, double pTmax) {
    
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::shower","begin --------------");
    }

    // Add new system, automatically with two empty beam slots.
    int iSys = partonSystemsPtr->addSys();
    
    // Verbose output
    if (verbose >= 8) 
      printOut("VinciaShower::shower","preparing to shower. System no. "
	     +num2str(iSys));
    
    // Loop over allowed range to find all final-state particles.
    Vec4 pSum;
    for (int i = iBeg; i <= iEnd; ++i) if (event[i].isFinal()) {
	partonSystemsPtr->addOut( iSys, i);
	pSum += event[i].p();
      }
    partonSystemsPtr->setSHat( iSys, pSum.m2Calc() );
    
    // Let prepare routine do the setup.    
    prepare(iSys, event);
    
    // Begin evolution down in pT from hard pT scale. 
    int nBranch = 0;
    do {
      double pTtimes = pTnext(event, pTmax, 0.);
      
      // Do a final-state emission (if allowed).
      if (pTtimes > 0.) {
	if (branch(event)) ++nBranch; 
	pTmax = pTtimes;
      }
      
      // Keep on evolving until nothing is left to be done.
      else pTmax = 0.;
    } while (pTmax > 0.); 
    
    // Return number of emissions that were performed.
    return nBranch;
    
  }
  
  //*********
  
  // Prepare system for evolution.
  
  void VinciaShower::prepare(int iSys, Event& event, bool limitPTmaxIn) {
    
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::prepare","begin --------------");
    }

    //If verbose, list event before showering
    if (verbose >= 4) {
      if (verbose >= 6 || iSys == 0) event.list();
      printOut("VinciaShower::prepare","preparing system no. "
	       +num2str(iSys)+" for showering. Size = "
	       +num2str(partonSystemsPtr->sizeOut(iSys))+
	       " limitPTmax = "+bool2str(limitPTmaxIn));
    }
    
    // Get generated event weight from Pythia.
    // Problem: for e+e- the first one is 1
    //          for dijets, the first one is 0. 
    //          how to tell the first time we are being called for an event?
    if (iSys <= 1) {
      setWeightAll(infoPtr->weight());
      for (int iVar=0; iVar<nVariations(); iVar++) {
	weightOld[iVar] = 0.0;
      }
    }

    // Reset dipole list for first interaction and for resonance decays.
    int iInA = partonSystemsPtr->getInA(iSys);
    if (iSys == 0 || iInA == 0) {
      branchelementals.resize(0);
      winners.resize(0);
    }

    // Vector to find total invariant mass of system
    Vec4 pSum;

    // Find and save all FS colors and anticolors
    map<int,int> indexOfAcol;
    map<int,int> indexOfCol;

    // Loop over event record. Find indices of FS particles with color lines
    int sizeSystem = partonSystemsPtr->sizeOut(iSys);
    for (int i = 0; i < sizeSystem; ++i) {    
      int i1 = partonSystemsPtr->getOut( iSys, i);
      // Skip if not present in final state
      if ( !event[i1].isFinal() ) continue;
      int col= event[i1].col();
      int acol= event[i1].acol();
      if (col != 0) indexOfCol[col]=i1;
      if (acol != 0) indexOfAcol[acol]=i1;
      if (col != 0 || acol != 0) pSum += event[i1].p();
    }

    // Store total invariant mass and set initial unordering scale
    mSystem  = m(pSum);
    Qrestart = mSystem;
    Q2UnOrdOld     = -1.;
    if (verbose >= 7) {
      printOut("VinciaShower::prepare",
	       "Total mass of showering system = "+num2str(mSystem));
    }

    // Now loop over colored particles 
    for (map<int,int>::iterator it=indexOfCol.begin(); 
	 it != indexOfCol.end(); ++it) {
      int col  = it->first;
      int i1   = it->second;
      int i2   = indexOfAcol[col];
      if (col == 0 || i1 == 0 || i2 == 0) continue;
      if (verbose >= 7) {
	printOut("VinciaShower::prepare",
		 "Creating antenna between "+num2str(i1,4)+", "
		 +num2str(i2,4)+": col = "+num2str(col,5)
		 +" ; scales = "+num2str(event[i1].scale())
		 +", "+num2str(event[i2].scale()));      
      }

      // Store dipole and set scale = invariant mass;
      Branchelemental trial(event,i1,i2,particleDataPtr);
      trial.old1=event[i1];
      trial.old2=event[i2];	        
      trial.system=iSys;

      // Save ancestor (for use in matching below)
      trial.iAncestor=event[i1].mother1();
      // If not from resonance decay, set mother to zero
      // (should be augmented to look further back?)
      if (event[i1].mother2() != 0 && event[i1].mother2() != trial.iAncestor)
	trial.iAncestor = 0;
      if (event[i2].mother1() != trial.iAncestor)
	trial.iAncestor = 0;
      if (event[i2].mother2() != 0 && event[i1].mother2() != trial.iAncestor)
	trial.iAncestor = 0;
      if (trial.iAncestor != 0) 
	if ( ! event[trial.iAncestor].isResonance() ) trial.iAncestor=0;
      if (trial.iAncestor != 0) {
	isResonance = true;
      }

      // Determine starting scale
      if ( ! useScale ) {
	trial.Qold=m(trial.old1,trial.old2);      
	if (!isResonance) {
	  if (event[trial.iAncestor].colType() != 0) {
	    Q2UnOrdOld  = trial.Qold;
	    pT2UnOrdOld = trial.Qold;
	    mD2UnOrdOld = trial.Qold;
	    if (evolutionMode >= 2) trial.unOrdering = true;
	  }
	}
      } else {
	// In VINCIA's normalization, multiply by 2
	trial.Qold=2.*min(trial.old1.scale(),trial.old2.scale());
	// If ancestor has color, use current scale as unordering scale
	if (!isResonance) {
	  if (event[trial.iAncestor].colType() != 0) {
	    Q2UnOrdOld  = trial.Qold;
	    pT2UnOrdOld = trial.Qold;
	    mD2UnOrdOld = trial.Qold;
	    if (evolutionMode >= 2) trial.unOrdering = true;
	  }
	}
      }
      trial.hasTrial=false;
      
      // Find neigbor on i1 side
      if (event[i1].acol() != 0) {
	int colL = event[i1].acol();
	trial.iL = indexOfCol[colL];	
	// Find next-to-nearest neigbor on i1 side
	if (trial.iL != 0 && event[trial.iL].acol() != 0) {
	  int colLL = event[trial.iL].acol();
	  trial.iLL = indexOfCol[colLL];	
	}
      }

      // Find neigbor on i2 side
      if (event[i2].col() != 0) {
	int colR = event[i2].col();
	trial.iR = indexOfAcol[colR];	
	// Find next-to-nearest neigbor on i1 side
	if (trial.iR != 0 && event[trial.iR].col() != 0) {
	  int colRR = event[trial.iR].col();
	  trial.iRR = indexOfAcol[colRR];	
	}
      }

      branchelementals.push_back(trial);
      
    }
    
    // Initialize nEmit and nSplit (include possible non-Born starting points)
    nEmit  = max(0,int(branchelementals.size())-1);
    nSplit = max(0,sizeSystem-2-nEmit);    

    // ME-MATCHING: first determine class of res-decay matrix element
    MEClass = 0;
    if (matching && int(branchelementals.size())>0 && isResonance) {
      int iRes    = branchelementals[0].iAncestor;
      int colType = event[iRes].colType();
      int chgType = event[iRes].chargeType();
      int spnType = event[iRes].spinType();
      // Colorless Resonances
      if (colType == 0) {
	// Colorless Vector Resonances
	if (spnType == 3) {
	  if (chgType == 0) MEClass = 23;  // Q=0, S=1, C=Singlet
	  else if (abs(chgType) == 1) MEClass = 24; // Q=1, S=1, C=Singlet	    
	}
      }
      // No other types implemented so far.
    }
    
    //If verbose output, list dipoles
    if (verbose >= 5) list();    
    
  }

//*********

// Select next pT in downwards evolution of the existing dipoles.

double VinciaShower::pTnext(Event& event, double pTbegAll, double pTendAll) {

  // Verbose output
  if (verbose >= 6) {
    printOut("VinciaShower::pTnext","begin --------------");
  }
    
  // Translate to VINCIA normalization: Q_max = s_max = 2pTevol_max
  double Qold = 2.*pTbegAll;

  // Initialize winner scale
  double Qwin = 0.;

  // Lowest phase space point
  double QMin = max(QMinAll,2.*pTendAll);

  // Verbose output
  if (verbose >= 4) {
    printOut("VinciaShower::pTnext","(re)starting evolution at scale Q = "
	     +num2str(Qold));
    if (verbose >= 6)  list();
  }

  // TMP STOP AFTER 4 branchings
  //if (branchelementals.size() >= 5) return 0.0;

  // Loop over dipole-antennae
  for (int iDip = 0; iDip < int(branchelementals.size()); ++iDip) {
    
    // Shorthand for current dipole-antenna
    Branchelemental* trial=&branchelementals[iDip];

    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::pTnext","* processing antenna no"+num2str(iDip,4)
	       +" col = "+num2str(trial->old1.col()));
    }

    // Generate new trial branchings, starting from Qold
    double mDip = m(event[trial->iOld1],event[trial->iOld2]);
    double Qnew = min(Qold,mDip);
    if (trial->unOrdering) Qnew = min(mDip,Qrestart);

    // Check if any available phase space
    if (Qnew <= QMin) {
      if (verbose >= 7) {
	printOut("VinciaShower::pTnext","| Qbegin = "
		 +num2str(Qnew)+" < QMin = "+num2str(QMin)+". Qnew = 0.0");
      }
      Qnew = 0.0;
      // Set antenna as being ready for hadronization
      trial->hasTrial       = true;
      trial->Qnew           = 0.0;
      trial->branchFlavor     = 0;
    } 

    // Lookup saved Qnew if there is one
    else if (trial->hasTrial) {
      Qnew = trial->Qnew;
      if (trial->branchFlavor == -1) Qnew = 0.0;
      if (verbose >= 8) {
	printOut("VinciaShower::pTnext","| Trial QE = "+num2str(Qnew)+
		 " retrieved");
      }
    }
    
    // Generate new trial scale
    else {
      
      // Set starting scale and generate a new Qnew from there
      // v.1024: Delegated this step to a separate function, generateQnew()
      //         (with the intention of making "pseudoshowers" possible)
      trial->Qnew = Qnew;
      Qnew = generateQnew(trial);
      
    }
      
    // Did I win?
    if (Qnew >= Qwin || Qwin <= 0.0) {      
      winner = trial;
      Qwin=Qnew;
    } 
    
    // Verbose output
    if (verbose >= 7) {                    
      printOut("VinciaShower::pTnext","> Qnew   = "
	       +num2str(Qnew)+"; type "+num2str(trial->branchFlavor,3)
	       +(iDip+1 == int(branchelementals.size()) ?
		 "; final Qwin = ":"; current Qwin = ")
	       +num2str(Qwin));  
    }  
  }

  // Verbose output  
  if (verbose >= 5 && Qwin > 0.) {
    if (verbose >= 7)  list();
    printOut("VinciaShower::pTnext","=== Winner at scale Qnew = "+num2str(Qwin)
	     +" antenna type "+num2str(winner->branchFlavor,3)
	     +"   col = "+num2str(winner->old1.col(),5));
  }

  // If winner was unordered type, update current restart scale for unordering
  if (winner->unOrdering) Qrestart = Qwin;

  // Test for end-of-shower condition
  if (Qwin <= QMin)  {

    // Verbose output
    if (verbose >= 5) {
      printOut("VinciaShower::pTnext","=== All trials now below cutoff QMin = " 
				+ num2str(QMin,3) + ".");
      Qwin = 0.0;
      if (verbose >= 6) {
	printOut("VinciaShower::pTnext","Final configuration was:");
	event.list();
      }
    }

    // The procedure below is slightly unstable.
    // Should find if it is possible to check whether a given system is
    // the last system to be considered and then update.

    // Add weight correction to cumulative sum (!= w0 if more than one system)
    double w0 = weight();
    weightSumSave[0]  += w0 - weightOld[0];
    weightSum2Save[0] += pow2(w0) - pow2(weightOld[0]);

    // For first system, add one to number of events. 
    if (weightOld[0] == 0.0) {
      nTotWeights++;
      nUnityWeight++;
    }

    // Check for nonunity weights
    if (abs(abs(w0)-1.0) > TINY && 
	(abs(abs(weightOld[0])-1.0) < TINY || weightOld[0] == 0)) {
	nNonunityWeight++;
	nUnityWeight--;
    }

    // Check for negative weights
    if (w0 < 0.0 && weightOld[0] >= 0.0) nNegativeWeight++;

    // Update old weight for the USR antennae
    weightOld[0] = w0;

    // Save uncertainty band weights
    if (uncertaintyBands) {
      for (int iVar=1; iVar<nVariations(); iVar++) {
	// Add weiht correction to cumulative sum 
	weightSumSave[iVar]  += weight(iVar) - weightOld[iVar];
	weightSum2Save[iVar] += pow2(weight(iVar)) - pow2(weightOld[iVar]);
	// Check for new max weight
	if (weightSave[iVar] > weightMaxSave[iVar]) 
	  weightMaxSave[iVar] = weightSave[iVar];
	// Update old weights
	weightOld[iVar] = weight(iVar);
      }      
    }
  }

  // Return nonvanishing value if branching found
  // Translate back to PYTHIA normalization --> factor 2 (see above).
  // If unordered branching won, return input scale - this should make 
  // the overall evolution still proceed in an ordered way.
  return min(pTbegAll,max(0.0,Qwin/2.0)); 
  
}

  //*********
  
  bool VinciaShower::branch(Event& event, bool isInterleaved) {
    
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::branch","begin --------------");
    }
    
    // Extract current QE and sDip scales
    double Qnew  = winner->Qnew;
    double Q2new = pow2(Qnew);
    double sDip  = winner->s();

    // Consistency check 1
    if ( sDip < TINY || Q2new < TINY) {
      if (verbose >= 3) {
	printOut("VinciaShower::branch",
		 "Warning: s ="+num2str(sDip)+"  Q2 = "+num2str(Q2new));
      }
      winner->nFailed++;
      if (winner->nFailed >= 50) {
	winner->Qnew = 0.0;
	winner->Qold = 0.0;
      }
      return false;
    }
  
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::branch","* Processing Branching at scale Q = " 
	       +num2str(Qnew)+"   isInterleaved = "+bool2str(isInterleaved));
    }

    //------------------------------------------------------------------------
    // GENERATE FULL TRIAL KINEMATICS and check whether it is accepted
    // v.1024: delegated this step to a separate function, acceptTrial()
    //         (with the intention of making "pseudoshowers" possible)
    if (not acceptTrial(event,winner)) {
      // Return: this trial branching did not happen
      return false;
    }
    //------------------------------------------------------------------------

    // Put new particles into the event record.
    int iOld1 = winner->iOld1;
    int iOld2 = winner->iOld2;
    int iNew1 = event.append(winner->new1);
    int iNew2 = event.append(winner->new2);
    int iNew3 = event.append(winner->new3);

    // Mark original dipole partons as branched and set daughters/mothers.
    event[iOld1].statusNeg();
    event[iOld1].daughters(iNew1, iNew3); 
    event[iOld2].statusNeg();
    event[iOld2].daughters(iNew1, iNew3);
    
    // Finally update the list of systems.
    partonSystemsPtr->replace(iSysSel, iOld1, iNew1);  
    partonSystemsPtr->addOut(iSysSel, iNew2);
    partonSystemsPtr->replace(iSysSel, iOld2, iNew3); 
    
    // Update number of branchings
    if (winner->branchFlavor == 0) nEmit++;
    else nSplit++;
    
    // Store old neighbours
    int iL  = winner->iL;
    int iLL = winner->iLL;
    int iR  = winner->iR;
    int iRR = winner->iRR;
    
    // Update old and add new dipoles
    if (verbose >= 5) {
      printOut("VinciaShower::branch",
	       "=== Branching Accepted. Updating dipole-antenna(e)");
    }
    if (winner->new1.colType() > 0) {        
      // Normal case, where iNew1-iNew2 has an LC color connection
      winner->reset(event,iNew1,iNew2,particleDataPtr);
      winner->old1 = event[iNew1];
      winner->old2 = event[iNew2];
      winner->Qold = min(winner->m(),Qnew);
      if (evolutionMode == 0 || evolutionMode >= 2) {
	winner->unOrdering = true;
      }
      // Update neighbours
      winner->iL=iL;
      winner->iLL=iLL;
      winner->iR=iNew3;
      winner->iRR=iR;

      // If second parton is octet, insert new antenna (iNew2,iNew3)
      if (event[iNew2].colType() > 0) {
	
	// Define new dipole-antenna
	Branchelemental trial(event,iNew2,iNew3,particleDataPtr);
	trial.system = iSysSel;
	trial.old1   = event[iNew2];
	trial.old2   = event[iNew3];	 
	trial.Qold   = min(trial.m(),Qnew);
	if (evolutionMode == 0 || evolutionMode >= 2) {
	  trial.unOrdering = true;
	}
	
	// Set tri- and quadrupole neighbors
	trial.iL       = iNew1;
	trial.iLL   = iL;
	trial.iR      = iR;
	trial.iRR = iRR;

	// Keep family name for matching
	trial.iAncestor=winner->iAncestor;
	
	// Add dipole-antenna to list, insert after iNew1-Inew2 to keep order
	for (int iDip=0; iDip<int(branchelementals.size()); iDip++) {
	  if (&branchelementals[iDip] == winner) {
	    branchelementals.insert(branchelementals.begin()+iDip+1,trial);      
	    break;
	  }
	}
	
      }
      
    } else {
      // Special case for qbar q-x final state, where only second pair is dipole
      winner->reset(event,iNew2,iNew3,particleDataPtr);
      winner->old1        = event[iNew2];
      winner->old2        = event[iNew3];
      winner->Qold        = min(winner->m(),Qnew);
      if (evolutionMode == 0 || evolutionMode >= 2) {
	winner->unOrdering     = true;
	Qrestart          = max(Qrestart,winner->m());
      }
      // Update neighbors
      winner->iL  = iNew1;
      winner->iLL = iL; 
      winner->iR  = iR;
      winner->iRR = iRR;
    }
    
    // Update neighbor pointers in other dipoles and unordering scales
    Qrestart         = mSystem;
    pT2UnOrdOld      = pow2(mSystem);
    mD2UnOrdOld      = pow2(mSystem);
    int uoType = 1;
    if (evolutionMode == 2) uoType = evolutionType;
    Q2UnOrdOld = pT2UnOrdOld;
    if (uoType == 2) Q2UnOrdOld = mD2UnOrdOld;

    // Loop over neighbors
    for (int iDip = 0; iDip < int(branchelementals.size()); iDip++) {
      
      // Shorthand 
      Branchelemental* ant = &branchelementals[iDip];
      
      // If sector, restart everyone from CM
      if (sectorOrdering && (evolutionMode == 0 || evolutionMode >= 2)) {
	ant->setTrial(false);
	ant->Qold = max(Qnew,ant->m());
	ant->unOrdering = true;
      }

      // If unordering, restart everyone from CM
      if (evolutionMode >= 2 || evolutionMode == 0) ant->unOrdering = true;
      
      // Update current min(pT) scale for unordering
      // (antennae already updated, so can use iOld values)
      if (ant->iL != 0) {
	pT2UnOrdOld = min(pT2UnOrdOld, RPtr->Q2E(event[ant->iL],event[ant->iOld1],
						 event[ant->iOld2],1));
	mD2UnOrdOld = min(mD2UnOrdOld, RPtr->Q2E(event[ant->iL],event[ant->iOld1],
						 event[ant->iOld2],2));
      }
      if (ant->iR != 0) {
	pT2UnOrdOld = min(pT2UnOrdOld,RPtr->Q2E(event[ant->iOld1],event[ant->iOld2],
						event[ant->iR],1));
	mD2UnOrdOld = min(mD2UnOrdOld,RPtr->Q2E(event[ant->iOld1],event[ant->iOld2],
						event[ant->iR],2));
      }
      // Default is pT-unordering, but allow mD-type also
      Q2UnOrdOld = pT2UnOrdOld;
      if (uoType == 2) Q2UnOrdOld = mD2UnOrdOld;

      // The post-branching branchelementals
      if (ant->iOld1 == iNew1 || ant->iOld1 == iNew2) {
	// No update necessary for this pair (handled above)
	continue;
      }    
      
      // Next-to-next-to-nearest neighbour update
      if (ant->iLL == iOld2) branchelementals[iDip].iLL   = iNew3; 
      if (ant->iRR == iOld1) branchelementals[iDip].iRR = iNew1; 
      
      // Next-to-nearest neighbour update 
      if (branchelementals[iDip].iL  == iOld2) {
	branchelementals[iDip].iL       = iNew3; 
	branchelementals[iDip].iLL   = iNew2;
      }
      if (branchelementals[iDip].iR == iOld1) {
	branchelementals[iDip].iR      = iNew1; 
	branchelementals[iDip].iRR = iNew2; 
      }
      
      // Nearest neighbour update

      if (branchelementals[iDip].iOld1 == iOld2) {
	branchelementals[iDip].setTrial(false);
	branchelementals[iDip].nFailed     = 0;
	branchelementals[iDip].iOld1       = iNew3;
	branchelementals[iDip].old1        = event[iNew3];
	branchelementals[iDip].iL       = iNew2;
	branchelementals[iDip].iLL   = iNew1;
	branchelementals[iDip].Qold        = min(branchelementals[iDip].Qold, 
						 branchelementals[iDip].m());
      }
      if (branchelementals[iDip].iOld2 == iOld1) {
	branchelementals[iDip].setTrial(false);
	branchelementals[iDip].nFailed     = 0;	
	branchelementals[iDip].old2        = event[iNew1];
	branchelementals[iDip].iOld2       = iNew1;
	branchelementals[iDip].iR      = iNew2;
	branchelementals[iDip].iRR = iNew3;
	branchelementals[iDip].Qold        = min(branchelementals[iDip].Qold, 
						 branchelementals[iDip].m());
      }
      
    }
    
    if (verbose >= 6) {
      event.list();
      list();
    }
    
    // Done.      
    return true;
    
  }
  
  //*********
  
  // Update list of branchelementals after ISR radiation
  
  void VinciaShower::update( int iSys, Event& event) {
    if (verbose >= 1)
      printErr("VinciaShower::update","system "+num2str(iSys)
	       +" (function not implemented)");
    if (verbose >= 5) event.list();
  }
  
//*********
  
// Update list of branchelementals after MPI rescattering 
  
  void VinciaShower::rescatterUpdate( int iSys, Event& event) {
    if (verbose >= 1)
      printErr("VinciaShower::rescatterUpdate","system "+num2str(iSys)
	       +" (function not implemented)");
    if (verbose >= 5) event.list();
  }
  
  //*********
  
  // Update list of branchelementals after MPI rescattering 
  
  bool VinciaShower::rescatterPropagateRecoil( Event& event, Vec4& pNew) {
    if (verbose >= 1)
      printErr("VinciaShower::rescatterPropagateRecoil","function not implemented");
    if (verbose >= 5) event.list();
    if (verbose >= 9) printOut("VinciaShower::rescatterPropagateRecoil",
			       "pNew = "+num2str(pNew.px())+","+num2str(pNew.py())
			       +","+num2str(pNew.pz())+","+num2str(pNew.e()));
    return false;
  }

//**********************************************************************
  
// END OF INHERITED FUNCTIONS FOR INTERFACE TO PYTHIA8
// BEGIN VINCIA INTERNAL FUNCTIONS
  
//**********************************************************************

// VinciaShower class member functions

//*********

// Initialize internal and diagnostics histograms

void VinciaShower::initVinciaHistos() {

  // Verbose output
  if (verbose >= 7) {
    printOut("VinciaShower::initVinciaHistos","begin --------------");
  }

  vinciaHistos.clear();

  // Don't book diagnostics histograms if verbose output not switched on.
  if (verbose <= 1) return;

  // Global
  int nBin;
  double binWid, wid, c, xMin, xMax;

  // MATCHING WEIGHTS
  nBin   = 61;
  binWid = 0.1;
  wid    = nBin * binWid;
  c      = 0.0;
  xMin   = c - wid/2.;
  xMax   = c + wid/2.;
  Hist HPME2LogUSR("Matching weight (USR)",nBin,xMin,xMax);

  // Histos for Paccept : gluon emission
  vinciaHistos["1stOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;

  // Histos for Paccept : gluon splitting
  vinciaHistos["1stOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;

  // Histos for PME : gluon emission
  vinciaHistos["1stOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;

  // Histos for PME : gluon splitting
  vinciaHistos["1stOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;

}

  //*********
  
  // Phase space regions
  
  // MR 11.10.2010: could return zero if the mass of a heavy quark was
  // set to zero in the configuration file, returns >= QMinAll now
  double VinciaShower::getQMin(double Q) {
    double QMinNow = QMinAll;
    if (Q > mt) QMinNow = mt;
    else if (Q > mtb) QMinNow = mtb;
    else if (Q > mb) QMinNow = mb;
    else if (Q > mc) QMinNow = mc;
    return max(QMinNow, QMinAll);
  }

  double VinciaShower::getQMin(int Nf) {
	double QMinNow = QMinAll;
    if (Nf >= 6) QMinNow = mt;
    else if (Nf == 5) QMinNow = mb;
    else if (Nf == 4) QMinNow = mc;
    return max(QMinNow, QMinAll);
  }
  // TODO: getQMax returns zero if the mass of a heavy quark was set to
  // zero in the configuration file, has to be corrected
  double VinciaShower::getQMax(int Nf) {
    if (Nf >= 6) return 1.e99;
    else if (Nf == 5) return mt;
    else if (Nf == 4) return mb;
    else return mc;
  }

  int VinciaShower::getNf(double Q) {
    if (Q > mt) return 6;
    else if (Q > mb) return 5;
    else if (Q > mc) return 4;
    else return 3;    
  }

  //*********
  
  // Definition of perturbative region : perturbativeTheta(a,1,b); a,1,b ordered.
  // MR 12.8.2010: changed to sij = 2*pi*pj and s
  // Input: s				= (p1+p2+p3)^2
  //        s12tilde		= 2*p1*p2
  //        s23tilde		= 2*p2*p3
  
  bool VinciaShower::perturbativeTheta(double s,double s12tilde,double s23tilde) {
    if (s <= 0.0) return false;
    else return RPtr->yE(s12tilde/s,s23tilde/s,cutoffType)*s > pow2(cutoffScale) ? 
	   true : false;}
  
  // Copy of perturbativeTheta using Branchelemental as input
  bool VinciaShower::perturbativeTheta(Branchelemental* trial) {
    // trial->s12 returns 2*p1*p2
    double s12=trial->s12();
    double s23=trial->s23();
    double s=trial->s();
    return perturbativeTheta(s,s12,s23);
  }

  // Copy of perturbativeTheta using Vec4 as input
  bool VinciaShower::perturbativeTheta(Vec4 pa, Vec4 pr, Vec4 pb) {
	// MR 12.8.2010: changed to use sartilde = 2*pa*pb and s = (pa+pr+pb)^2
    double sartilde=2*pa*pr;
    double srbtilde=2*pr*pb;
    double s		=m2(pa+pr+pb);
    return perturbativeTheta(s,sartilde,srbtilde);
  }
 
//*********

// Generate a new trial scale, starting from trial->Qnew

  double VinciaShower::generateQnew(Branchelemental* trial, double Qstart) {

    // Default: use trial->Qnew as starting scale
    double Qnew = trial->Qnew;
    if (Qstart >= 0.0) Qnew = Qstart;     

    // ID and mass of parents
    int     id1  = trial->old1.id();
    int     id2  = trial->old2.id();
    double  sDip = trial->s();    
    double  mDip = sqrt(sDip);

    // Highest possible branching scale is mass of mother dipole
    Qnew = min(Qnew,mDip);
    
    // Sanity check
    if (Qnew <= QMinAll) return 0.0;

    // Loop over evolution regions (nf, PS boundaries)
    bool acceptQnew      = false;
    while (not acceptQnew) {
	
      // Reset hasTrial
      trial->hasTrial      = false;

      // Determine region for alphaS running and Z overestimates
      // (when including massive quarks and actually matching 
      //  across flavor thresholds, we may have to be more careful)
      int nfNow            = getNf(Qnew);
      int nGluonToQuarkNow = min(nfNow,nGluonToQuark);
      double QMinNow       = getQMin(Qnew);
	
      // Verbose output
      if (verbose >= 7) {
	printOut("VinciaShower::generateQnew","| Qbegin = "+num2str(Qnew)
		 +" QMinNow = "
		 +num2str(QMinNow)+" nfNow = "+num2str(nfNow)
		 +" mDip = "+num2str(mDip));	
      }
      
      // Scaled Q2min 
      // (also for massive case)
      double yEmin = pow2(QMinNow/mDip);
      
      // AlphaS factor
      double alphaFac = alphaSvalue;
      if (alphaSorder == 1) {	  
	if (alphaSmode == 2) {
	  // Running alphaS outside integral.
	  alphaFac = alphaS.alphaS(pow2(alphaSscaleFactor*mDip));
	} else if (alphaSmode == 3) {
	  // pT as scale with non-pT as evolution variable
	  // generate trials with constant alphaSMax
	  alphaFac = alphaSMax;
	} else {
	  // Running alphaS inside integral; factor 1/b0.
	  alphaFac = 12.0*M_PI/(11.0*CA-2.0*nfNow);
	}	  
      }
      
      // Factor 1 / 4pi from coupling / phase-space 
      alphaFac /= 4. * M_PI;
      
      // MR 12.7.2010: need to multiply NormEmit and NormSplit by a factor
      // Ecm / sqrt( lambda( Ecm^2, m_parent1^2, m_parent2^2 ) )
      // with Lambda the Kallen function, to account for the normalisation 
      // to the two-particle phase space volume
      double s1 = trial->old1.m2();
      double s2 = trial->old2.m2();
      double phasespaceVol = 1.0;
      if (s1 > TINY || s2 > TINY) {
	double kallenFac = sqrt(pow2(sDip - s1 - s2) - 4*s1*s2);
	phasespaceVol = sDip/kallenFac;
	if (verbose>=8 && kallenFac > 1) {
	  printOut("VinciaShower::pTnext", "massive phase space volume factor is "
		   + num2str( phasespaceVol ) );
	}
      }
      
      //**********************************************************************
      // Hull: defines phase space region within which to generate trials
      //**********************************************************************
      
      // Gluon Emission
      // Z Boundaries and normalization factor for Z integrals
      double zMin  = 0.5;
      double zMax  = 0.5;
      double zNorm = 1.0; // zNorm = 2kQ
      if (evolutionType == 1) {
	// pT-ordering (normalization 2*kQ = 1)
	zMin  = 0.5*(1.0-sqrt(1.0-yEmin)); 
	zMax  = 0.5*(1.0+sqrt(1.0-yEmin)); 
      } else if (evolutionType == 2) {
	// mAnt-ordering (normalization 2*kQ = 2)
	zMin  = yEmin/2.; 
	zMax  = 1.-yEmin/2.;
	zNorm = 2.0;
      } else if (evolutionType == 3) {
	// Energy ordering (2*kQ = 1, z limits imposed by hadronization cut)
	double yCut = pow2(cutoffScale/mDip);
	if (cutoffType == 1) {
	  zMin  = 0.5*(1.0-sqrt(1.0-yCut));
	  zMax  = 0.5*(1.0+sqrt(1.0-yCut));
	} else {
	  zMin  = yCut/2.;
	  zMax  = 1.-yCut/2.;
	} 
      } else if (evolutionType == 4) {
	// Anti-collinear VS-ordering (normalization 2*kQ = 4)
	zMin  = 0.5*(1.-pow2(1.0-yEmin));
	zMax  = 0.5*(1.+pow2(1.0-yEmin));
	zNorm = 4.0;
      } else if (evolutionType == 5) {
	// Collinear-safe energy ordering (2*kQ = 1, QE -> pT close to collinear)
	zMin  = 0.5*(1.-pow(1.0-pow2(yEmin),0.25));
	zMax  = 0.5*(1.+pow(1.0-pow2(yEmin),0.25));
      }
      double zIntEmit = zNorm * log(zMax*(1.-zMin)/zMin/(1.-zMax));
      // Color Factor * AlphaS/4pi * Z Integral
      // = 1/b for const aS or = 1/b' for QE-dep running aS.
      double NormEmit = CA * alphaFac * zIntEmit * phasespaceVol;
      // If doing matching:
      // Multiply by headRoomFactor for emit-type antennae at this order
      if (nEmit + nSplit + 1 <= matchingOrder) {
	NormEmit *= headRoomFactorEmit[nEmit+nSplit+1];
	trial->setHeadRoomFactorEmit(headRoomFactorEmit[nEmit+nSplit+1]);
      } else {
	trial->setHeadRoomFactorEmit(1.0);
      }
            
      
      // Gluon Splitting:
      double NormSplit(0.0);
      if ( (id1 == 21 || id2 == 21) && nGluonToQuark >= 1) {
	// Use 1/(sqqbar+2mq2) as trial function and its inverse as evol var.
	// (where sqqbar = dot product = (pq + pqbar)^2 - 2mqq^2)
	zMax = 1.-yEmin;
	double zIntSplit = zMax;
	NormSplit = nGluonToQuarkNow * TR * alphaFac * zIntSplit * phasespaceVol;
	if (id1 == 21 && id2 == 21) NormSplit *= 2.; 
	// If doing matching:
	// Multiply by headRoomFactor for split-type antennae at this order
	if (nEmit + nSplit + 1 <= matchingOrder) {
	  NormSplit *= headRoomFactorSplit[nEmit+nSplit+1];
	  trial->setHeadRoomFactorSplit(headRoomFactorSplit[nEmit+nSplit+1]);
	} else {
	  trial->setHeadRoomFactorSplit(1.0);
	}
      }
      
      // Generate Qnew 
      if (alphaSorder == 0 || alphaSmode == 2 || alphaSmode == 3) {
	// Running outside trial integral (extra 1/2 from square root)
	Qnew = Qnew * pow(rndmPtr->flat(),1./(NormEmit+NormSplit)/2.);
      } else {
	// Running inside integral 
	double Lambda = Lambda5flav;
	if (nfNow == 4) Lambda = Lambda4flav;
	if (nfNow == 3) Lambda = Lambda3flav;
	double expn = pow(rndmPtr->flat(),1./(NormEmit+NormSplit));
	Qnew = Lambda/alphaSscaleFactor * pow(alphaSscaleFactor*Qnew/Lambda,expn);
      }
      
      // Check whether Region crossing occurred.
      if (Qnew >= QMinNow || QMinNow <= QMinAll) {
	
	// Qnew generated inside present region or no new region to go to.
	if (verbose >= 8) {
	  printOut("VinciaShower::pTnext","| trial QE = "+num2str(Qnew)+
		   " accepted with nf = "+num2str(nfNow));
	}
	
	// If below cutoff, store as zero, else store Qnew
	if (Qnew < QMinAll) Qnew=0.0;
	trial->Qnew     = Qnew;
	trial->hasTrial = true;
	acceptQnew = true;
	
	// Select between gluon emission and gluon splitting
	trial->branchFlavor     = 0;
	if (Qnew > 0. && NormSplit > 0.) {	    
	  double Ran = rndmPtr->flat()*(NormEmit + NormSplit) - NormEmit;
	  if (Ran > 0.) trial->branchFlavor = 
			  int(nGluonToQuarkNow * Ran/NormSplit + 1); 
	}
	
      } 
      
      // Crossed region, try again in next region
      else {
	
	// Verbose Output
	if (verbose >=8 ) {
	  printOut("VinciaShower::pTnext","=== Trial Rejected at QE = "
		   +num2str(Qnew)+" Crossed Region Boundary at QMinNow = "
		   +num2str(QMinNow)
		   +" (nf was "+num2str(nfNow)+")");
	}
	
	// Reset Qnew and force new trial generation
	Qnew            = QMinNow;
	acceptQnew     = false;
	
      }
      
      // Don't bother with accept/reject if Qnew already < TINY
      // (must be here, since a previous small Qnew could just be 
      //  a region crossing)
      if (Qnew < TINY) continue;
      
      // Freeze alphaS at alphaSMax (by veto if above alphaSMax)
      if (acceptQnew && alphaSorder == 1 && alphaSmode != 3) {
	double mu2;
	if (alphaSmode == 1) mu2 = pow2(Qnew*alphaSscaleFactor);
	else mu2 = pow2(mDip*alphaSscaleFactor);
	double aS = alphaS.alphaS(mu2); 
	if (aS > alphaSMax) {
	  double wAlphaS = alphaSMax / aS;
	  if (rndmPtr->flat() > wAlphaS) {
	    acceptQnew     = false; 
	    if (verbose >= 8) {
	      printOut("VinciaShower::pTnext",
		       "=== Trial Rejected at QE = "+num2str(Qnew)
		       +". alphaSTrial = "+num2str(aS)
		       +" > alphaSMax = "+num2str(alphaSMax));      
	    }
	  }
	}
      }

      // For g -> massive quarks, check that minimal phase space open
      if (trial->branchFlavor == 4 && Qnew - 2.*mc < 1. ) acceptQnew = false;
      if (trial->branchFlavor == 5 && Qnew - 2.*mb < 1. ) acceptQnew = false;
      if (trial->branchFlavor == 6 && Qnew - 2.*mt < 1. ) acceptQnew = false;
      
    }

    // Return Qnew
    return Qnew;

  }
  
  // ******
  // 
  // acceptTrial : main function to accept or reject a trial branching 
  //               assumes branching scale trial->Qnew already generated
  // Steps:
  //     *) Generate second branching invariant, given Qnew
  //        Construct new partons, compute basic (LL) accept probability,
  //        and construct kinematics in LAB frame
  //     *) Optional: check sector ordering (if doing sector showers) 
  //     *) Optional: check alternative hadronization vetos
  //     *) Optional: apply beyond-LL (beyond 2->3) improvements 
  //     *) Set mother and color-flow information
  //     *) Optional: match to fixed-order QCD matrix elements (from MadGraph)
  //     *) Optional: post-processing of uncertainty variations
  //     *) Accept or Reject Trial Emission
  // 
  //        Return true if trial accepted, false otherwise
   
  bool VinciaShower::acceptTrial(Event& event, Branchelemental* trial) {

    // Global quantities
    double Qnew = trial->Qnew;

    //------------------------------------------------------------------------
    // *) Generate branching 
    // Return if invariants outside physical phase space
    if (not generateBranching(winner)) {
      winner->setTrial(false);    
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial failed generateBranching. Returning."); 
      }
      return false;
    } else {
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial passed generateBranching. Continuing...");
      }      
    }
    
    //------------------------------------------------------------------------
    // *) Optional: check sector ordering (if doing sector showers) 
    if (sectorOrdering && !sectorAccept(event,winner)) {
      winner->setTrial(false);
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial failed sector veto. Returning.");
      }
      return false;
    } else if (sectorOrdering) {
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial passed sector veto. Continuing...");
      }
    }      
    
    //------------------------------------------------------------------------
    // *) Optional: check alternative hadronization vetos
    bool hadAccept = true;    
    // Check no QCD invariant mass among neighbors below pion mass
    // (in more sophisticated future, check only color-connected pairs)
    double m2Min = m2(winner->new1,winner->new2);
    m2Min = min(m2Min,m2(winner->new2,winner->new3));
    if (winner->iL != 0) m2Min = min(m2Min,m2(event[winner->iL],winner->new1));
    if (winner->iR != 0) m2Min = min(m2Min,m2(event[winner->iR],winner->new3));
    double m2pi0 = pow2(particleDataPtr->m0(111));
    if (m2Min < 1.1*m2pi0) hadAccept = false;
    if (not hadAccept) {    
      winner->nFailed++;
      if (winner->nFailed < 50 && evolutionType != cutoffType) {
	winner->setTrial(false); 
      } else {
	// Switch off this branchelemental.
	winner->Qold = 0.0;
	winner->Qnew = 0.0;
      }
      if (verbose >= 5) {
	printOut("VinciaShower::branch"
		 ,"=== Trial failed neighbor hadronization veto. Returning.");
      }
      return false;
    } else {
      if (verbose >= 7) {
	printOut("VinciaShower::branch"
		 ,"Trial passed neighbor hadronization veto. Continuing...");
      }
    }
    
    //------------------------------------------------------------------------
    // *) Apply beyond-LL (beyond 2->3) improvements 
    //    Pimp   (allows suppressed unordered branchings)
    //    Pari   (ARIADNE trick for improving g->qqbar splittings)

    // Find initial particles in dipole branching.
    int iOld1 = winner->iOld1;
    int iOld2 = winner->iOld2;
    int acol1 = event[iOld1].acol();
    int col2  = event[iOld2].col();
    iSysSel = winner->system;
    int branchFlavor = winner->branchFlavor;
    
    // If smooth ordering, check if branching should be vetoed
    // Power-suppressed unordering 
    double Pimp = 1.0;
    if (evolutionMode >= 2 && winner->unOrdering) {
      // Compute 2*pT scale of new branching and compute Pimp factor
      double Q2UnOrd = pow2(Qnew);
      if (evolutionMode == 2 && evolutionType != 1)  
	Q2UnOrd = RPtr->Q2E(winner->new1,winner->new2,winner->new3,1);
      Pimp = 1.0/(1.0 + Q2UnOrd/Q2UnOrdOld);
      // Check for negative Pimp factor (normally should not happen!)
      if (Pimp < 0.0) {
	if (verbose >= 1) 
	  printErr("VinciaShower::branch","Pimp factor = "+num2str(Pimp)+" < 0");
	Pimp =0.0;
      }
      // Apply Pimp factor
      Paccept[0] *= Pimp;  
      // Keep track of uncertainty bands
      if (verbose >= 8) 
	printOut("vinciaShower::acceptTrial","PS Pimp = "+num2str(Pimp)
		 +" -> Paccept[0] = "+num2str(Paccept[0]));
    }
    
    // Matching: Which order are we currently at?
    int nEmitNow  = nEmit;
    int nSplitNow = nSplit;
    if (winner->new2.id() == 21) nEmitNow++; else nSplitNow++;
    bool doMatch = MEClass != 0 && nEmitNow + nSplitNow <= matchingOrder;

    // For gluon splitting to quarks, impose additional improvements
    double Pari = 1.0;
    double sP = m2(event[iOld1],event[iOld2]);
    // MR 9.3.2011: include option to turn Pari off
    if (
      branchFlavor != 0 && 
        settingsPtr->flag("VinciaShower:GluonSplittingCorrection")
    ) {

      // Impose ARIADNE factor for g->qqbar.
      //       Pari = 2/(1+sN/sP)    (sN = s(neighbor), sP = s(parent))
      // If s(neighbor) << s(parent) -> splitting suppressed (screened?)
      // If s(neighbor) >> s(parent) -> splitting enhanced (up to factor 2)      
      double sN;
      int iN;
      // Which side of dipole-antenna is splitting? 
      if (event[iOld1].id() != 21 || winner->new1.id() == 21) {
	iN = winner->iR;
	// Only apply if neighbor exists, is a quark, and is in final state 
	if (iN != 0 && event[iN].id() != 21 && event[iN].isFinal() ) {
	  sN = m2(event[iOld2],event[iN]);
	  Pari = 2.0*sN/(sP + sN);      
	}
      } else { 
	iN = winner->iL;
	// Only apply if neighbor exists, is a quark, and is in final state 
	if (iN != 0 && event[iN].id() != 21 && event[iN].isFinal() ) {
	  sN = m2(event[iN],event[iOld1]);	
	  Pari = 2.0*sN/(sP + sN);      
	}	  
      }
      // Check for negative Pari factor (normally should not happen!)
      if (Pimp < 0.0) {
	if (verbose >= 1) 
	  printErr("VinciaShower::branch","Pari factor = "+num2str(Pari)+" < 0");
	Pari = 0.0;
      }
      // Apply Pari factor
      Paccept[0] *= Pari;      

      if (verbose >= 8) 
	printOut("VinciaShower::trialAccept","PS Pari = "+num2str(Pari)
		 +" -> Paccept[0] = "+num2str(Paccept[0]));
      //      cout<< "   Pari = "<<Pari<<" -> Paccept[0] = "<<Paccept[0]<<endl;
      
      // DEVELOPMENT, switched off in distribution version
      // Beyond matched orders: match to 2>4 splitting function
      // (probably needs to be refined to deal with a gluon 
      //  splitting immediately after last matched order ...)
      /*
      if (!doMatch) {
	vector<Vec4> p2to4, p2to3, pClu;
	int colTypeA, colTypeB;
	// Splitting gluon on RIGHT side of dipole-antenna
	if (event[iOld1].id() != 21 || winner->new1.id() == 21) {
	  p2to4.push_back(winner->new1.p());
	  p2to4.push_back(winner->new2.p());
	  p2to4.push_back(winner->new3.p());
	  p2to4.push_back(event[winner->iR].p());
	  colTypeA = winner->old1.colType();
	  colTypeB = event[winner->iR].colType();
	} 
	// Splitting gluon on LEFT side of dipole-antenna
	else { 
	  p2to4.push_back(event[winner->iL].p());
	  p2to4.push_back(winner->new1.p());
	  p2to4.push_back(winner->new2.p());
	  p2to4.push_back(winner->new3.p());
	  colTypeA = event[winner->iL].colType();
	  colTypeB = winner->old2.colType();
	}
	
	// Compute full 2>4 accept probability
	double P2to4 = 1.0;
	double s01   = m2(p2to4[0],p2to4[1]);
	double s12   = m2(p2to4[1],p2to4[2]);
	double s23   = m2(p2to4[2],p2to4[3]);
	double s012  = m2(p2to4[0],p2to4[1],p2to4[2]);
	double s123  = m2(p2to4[1],p2to4[2],p2to4[3]);
	double s4    = m2(p2to4[0],p2to4[1],p2to4[2],p2to4[3]);
	if (colTypeA == 1 && colTypeB == -1) {
	  // Gluon splitting inside a qqbar>qgqbar parent dipole-antenna
	  // Treat as non-identical quarks. Check possibility to include
	  // identical-quark interferences as well.
	  P2to4 = B40(p2to4,0,1,2,3);
	  // History A
	  int kinTypeA = antennae[iqgSplit].getKineMapType();
	  VinClu(kinTypeA,p2to4,0,1,2,pClu);
	  double sA01  = m2(pClu[0],pClu[1]);
	  double sA12  = m2(pClu[1],p2to4[3]);
	  double termA = antennae[iqqEmit].a2(sA01/s4,sA12/s4)/s4;
	  termA *= antennae[iqgSplit].a2(s01/s012,s12/s012)/s4;
	  // History B
	  int kinTypeB = antennae[iqgSplit].getKineMapType();
	  VinClu(kinTypeB,p2to4,1,2,3,pClu);
	  pClu.push_back(p2to4[3]);
	  double sB01  = m2(p2to4[0],pClu[0]);
	  double sB12  = m2(pClu[0],pClu[1]);
	  double termB = antennae[iqqEmit].a2(sB01/s4,sB12/s4)/s4;
	  termB *= antennae[iqgSplit].a2(s23/s123,s12/s123)/s4;
	} else if (colTypeA == 1 && colTypeB == 2) {
	  // Gluon splitting inside a qg>qgg parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = E40(p2to4,0,1,2,3);
	} else if (colTypeA == 2 && colTypeB == -1) {
	  // Gluon splitting inside a gqbar>ggqbar parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = E40(p2to4,3,2,1,0);
	} else if (colTypeA == 2 && colTypeB == 2) {
	  // Gluon splitting inside a gqbar>ggqbar parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = G40(p2to4,3,2,1,0);
	}
      }
      */
      
    }

    //------------------------------------------------------------------------
    // *) Set mother and color-flow information

    // Set mothers
    winner->new1.mothers(iOld1,iOld2);
    winner->new2.mothers(iOld1,iOld2);
    winner->new3.mothers(iOld1,iOld2);
    
    // Set colour flow, deleting branched dipoles and adding new ones.
    winner->new1.acol(acol1);
    winner->new1.col(0);
    winner->new2.acol(0);
    winner->new2.col(0);
    winner->new3.acol(0);
    winner->new3.col(col2);   
    if (winner->new1.colType() > 0) {          
      int col=event.nextColTag();   
      winner->new1.col(col);
      winner->new2.acol(col);
    } 
    if (winner->new2.colType() > 0) {
      int col=event.nextColTag();
      winner->new2.col(col);
      winner->new3.acol(col);
    }    

    // Set masses
    // MR 17.7.2010: there is a problem with particles which are
    // off-shell by more than TINY, set the masses based on ID and
    // check off-shellness here
    // PS 23.2.2011: changed denominator in offshell to sP instead of Qnew
    // and modified linear to squared dependence, since that's really what
    // the numerical precision is driven by
    winner->new1.m( (vinComPtr->isMassive(winner->new1.id()) ?
		     winner->new1.m0() : 0.0) );
    winner->new2.m( (vinComPtr->isMassive(winner->new2.id()) ?
		     winner->new2.m0() : 0.0) );
    winner->new3.m( (vinComPtr->isMassive(winner->new3.id()) ?
		     winner->new3.m0() : 0.0) );
    double offshell1 = abs( winner->new1.m2() - winner->new1.p().m2Calc() );
    double offshell2 = abs( winner->new2.m2() - winner->new2.p().m2Calc() );
    double offshell3 = abs( winner->new3.m2() - winner->new3.p().m2Calc() );
    // Relative to branching scale (TINY should only measure ratios)
    offshell1 /= sP;
    offshell2 /= sP;
    offshell3 /= sP;
    // check off-shellness
    if (offshell1 > TINY || offshell2 > TINY || offshell3 > TINY) {
      if (verbose >= 3) {
	printOut("VinciaShower::acceptTrial", "new particle failed off-shell test");
	cout << "p1 =	" << winner->new1.p();
	cout << "p2 =	" << winner->new2.p();
	cout << "p3 =	" << winner->new3.p();
	cout << setprecision(16);
	cout << "TINY		= " << TINY			<< endl;
	cout << "offshell1	= " << offshell1	<< endl;
	cout << "offshell2	= " << offshell2	<< endl;
	cout << "offshell3	= " << offshell3	<< endl;
      }      
      return false;
    } 
	
    //------------------------------------------------------------------------
    // *) Match to fixed-order QCD matrix elements (from MadGraph)

    // Compute pT2 for this trial (to be used below)
    double pT2New = pow2(winner->Qnew)/4.;
    if (evolutionType != 1) {
      pT2New = 0.25*RPtr->Q2E(winner->new1,winner->new2,winner->new3,1);
    }

    // QCD order
    int nQCDNow = nEmitNow + nSplitNow;
	
    // ME Matching
    doMatch = doMatch && matchingPtr->isMatched(MEClass,nEmitNow,nSplitNow,isMGOn);

    // Check QCD order
    if (doMatch) doMatch = (nQCDNow <= matchingOrder);

    // Check MadGraph is switched on
    if (doMatch) doMatch = (isMGOn);

    // Construct momentum and id vectors for matching 
    vector<int> id;
    vector<Vec4> p;
    string HPME = "none";
    if (doMatch) {      
      if (verbose >= 8) 
	printOut("VinciaShower::acceptTrial","Testing Order "
		 +num2str(nEmitNow+nSplitNow)+" MG Matching ...");
      int iAncestor = winner->iAncestor;
      id.push_back(event[iAncestor].id());
      p.push_back(event[iAncestor].p());
      for (int iDip=0;iDip<int(branchelementals.size());iDip++) {
	Branchelemental* prevAnt=&branchelementals[iDip];
	if (prevAnt->iOld1 == winner->iOld1 && 
	    prevAnt->iOld2 == winner->iOld2) {
	  id.push_back(winner->new1.id());
	  id.push_back(winner->new2.id());
	  id.push_back(winner->new3.id());
	  p.push_back(winner->new1.p());
	  p.push_back(winner->new2.p());
	  p.push_back(winner->new3.p());
	} else {
	  if (prevAnt->iOld1 != winner->iOld2) {	  
	    id.push_back(event[prevAnt->iOld1].id());
	    p.push_back(event[prevAnt->iOld1].p());
	  }
	  if (event[prevAnt->iOld2].colType() != 2) {
	    id.push_back(event[prevAnt->iOld2].id());
	    p.push_back(event[prevAnt->iOld2].p());
	  }
	}
      }
    }

    // Decide whether to match over full phase space or impose matching scale
    bool imposeMatchingScale = false;
    double Q2Match=0.0;
    if (doMatch) {
      // Impose matching scale starting from matchingRegOrder
      if (nQCDNow >= matchingRegOrder && matchingRegOrder != 0) 
	imposeMatchingScale = true;
      // If matchingScaleType is relative, determine matching scale
      if (matchingScaleIsAbs) Q2Match = pow2(matchingScale);
      else Q2Match = pow2(matchingScaleRatio * mSystem);
    }
    
    // Compute matching scale dampening factor (if imposed)
    double DeltaDamp = 1.0;
    double Q2Damp     = pow2(mSystem);

    // Check matching scale
    if (doMatch && imposeMatchingScale) {		
      // Impose matching scale in 2*pT or sqrt(2)*m?
      if (matchingScaleType == 1) {
	Q2Damp = 4*pT2New;
      } else {
	Q2Damp = RPtr->Q2E(winner->new1.p(),winner->new2.p(),winner->new3.p(),2);
      }
      // Check color neigbors
      if (winner->iL != 0) {
	Q2Damp = min(Q2Damp,RPtr->Q2E(event[winner->iL].p(),winner->new1.p(),
				      winner->new2.p(),matchingScaleType));
      }
      if (winner->iR != 0) {
	Q2Damp = min(Q2Damp,RPtr->Q2E(event[winner->iR].p(),winner->new3.p(),
				      winner->new2.p(),matchingScaleType));
      }
      // MatchingRegShape = 0: Step Function
      if (matchingRegShape == 0) {
	if (Q2Damp < Q2Match) DeltaDamp = 0.0;
	else DeltaDamp = 1.0;
      } 
      // MatchingRegShape = 1: linear in log(Q) between 1/2 QM2 < Q2 < 2 QM2
      else {
	double log2R = log(Q2Damp/Q2Match)/log(2.);
	if (log2R < -1) {
	  DeltaDamp = 0.0;
	  doMatch = false;
	} else if (log2R > 1.) {
	  DeltaDamp = 1.0;
	} else {	    
	  DeltaDamp = 0.5*log2R + 0.5 ;
	}
      }
    }

      /*
      cout << " ids  =";
      for (int i=0;i<id.size();i++) cout<<" "<<id[i];      
      cout << " nMom = "<<p.size()<<endl;
      */
      // Get matching coefficients

    if (doMatch) {
      if (matchingPtr->getPME(1,id,p,PME)) {
	Paccept[0] *= 1.0 + DeltaDamp * (PME[0] - 1.); 
	if ((Paccept[0] > 1.0 + TINY) && verbose >= 3) {	  
	  cout <<" order = "<<nQCDNow<<"  unregulated PME = "<<PME[0]<<" at Qnew = "<<Qnew<<" mDamp = "<<sqrt(Q2Damp)<<" mDamp/mMatch = "<<sqrt(Q2Damp/Q2Match)<<" Pimp = "<<Pimp << " Pari = "<<Pari<<" DeltaDamp = "<<DeltaDamp<<endl;
	  cout <<"       ijk = "<<winner->new1.id()<<" "<<winner->new2.id()<<" "
	       <<winner->new3.id()
	       <<"  mij = "<<sqrt(m2(winner->new1.p()+winner->new2.p()))
	       <<" mjk = "<<sqrt(m2(winner->new2.p()+winner->new3.p()))
	       <<" mijk = "<<sqrt(sP);
	  cout <<"      yij = "<< m2(winner->new1.p()+winner->new2.p())/sP
	       <<" yjk = "<<m2(winner->new2.p()+winner->new3.p())/sP<<endl;
	  
	}
	if (uncertaintyBands) {
	  //	  cout<<" PACLL "<<Paccept[0]<<" "<<Paccept[1]<<" "<<Paccept[2]<<" "<<Paccept[3]<<endl;
	  for (int iVar=1; iVar<nVariations(); iVar++) {
	    Paccept[iVar] *= PME[iVar]/PME[0];      	
	  }
	  //cout<<" PME   "<<PME[0]<<" "<<PME[1]<<" "<<PME[2]<<" "<<PME[3]<<endl;
	  //cout<<" PACME "<<Paccept[0]<<" "<<Paccept[1]<<" "<<Paccept[2]<<" "<<Paccept[3]<<endl;
	}
	// Fill diagnostics histos?
	if (verbose >= 2) {
	  string matchingHisto="1stOrderMatchingLog10";
	  if (nQCDNow == 2) matchingHisto="2ndOrderMatchingLog10";
	  else if (nQCDNow == 3) matchingHisto="3rdOrderMatchingLog10";
	  else if (nQCDNow == 4) matchingHisto="4thOrderMatchingLog10";
	  else if (nQCDNow == 5) matchingHisto="5thOrderMatchingLog10";
	  else if (nQCDNow == 6) matchingHisto="6thOrderMatchingLog10";
	  else if (nQCDNow == 7) matchingHisto="7thOrderMatchingLog10";
	  string HPacc = matchingHisto+"P(ME/TrialAnt)";
	  if (nSplitNow == nSplit) HPacc += ":Emit";
	  else HPacc += ":Split";
	  vinciaHistos[HPacc].fill(log10(max(TINY,Paccept[0])));	  	
	  HPME = matchingHisto+"P(ME/PhysAnt)";
	  if (nSplitNow == nSplit) HPME += ":Emit";
	  else HPME += ":Split";
	}
      }
    }

    // If not doing matching ...
    else {

      // Do NLC corrections for gluon emissions
      if (showerNLC >= 1 && branchFlavor == 0) {

	if (verbose >= 7) printOut("VinciaShower::acceptTrial","applying NLC correction type "
				   + num2str(showerNLC));

	// First find index of current antenna
	int iThis(0);
	for (int iDip = 0; iDip < int(branchelementals.size()); iDip++) {
	    
	  Branchelemental* ant = &branchelementals[iDip];
	  if (ant->iOld1 == winner->iOld1 &&
	      ant->iOld2 == winner->iOld2) {
	    iThis = iDip;
	    break;
	  }
	}  
	int i1(iThis), i2(iThis);
	
	// Look for q and qbar ends containing this 
	bool isOK = true;
	bool isQ = (branchelementals[iThis].old1.colType() == 1);
	bool isQbar = (branchelementals[iThis].old2.colType() == -1);
	while (!isQ && i1 >= 0 && isOK) {
	  i1--;
	  // Stop if reached top of antenna list without finding endpoint
	  if (i1 < 0) isOK = false;
	  // Also stop if reached an antiquark (new chain)
	  else if (branchelementals[i1].old2.colType() == -1) isOK = false;
	  // Otherwise continue looking for quark end
	  else isQ = (branchelementals[i1].old1.colType() == 1) ? true : false;
	}
	while (!isQbar && i2 < int(branchelementals.size()) && isOK) {
	  i2++;
	  // Stop if reached end of antenna list without finding endpoint
	  if (i2 >= int(branchelementals.size())) isOK = false;
	  // Also stop if reached a quark (new chain)
	  else if (branchelementals[i2].old1.colType() == 1) isOK = false;
	  // Otherwise continue looking for antiquark end
	  isQbar = (branchelementals[i2].old2.colType() == -1) ? true : false;
	}
	isOK = isOK && isQ && isQbar;

	// If the emission happened inside an identifiable q-qbar chain:
	if (isOK) {

	  // Construct momenta
	  vector<Particle*> pC;
	  int iC(0);
	  for (int iDip = i1; iDip <= i2; iDip++) {
	    
	    // Shorthand
	    Branchelemental* ant = &branchelementals[iDip];

	    // For branching dipole, add new momenta instead of old ones
	    if (ant->iOld1 == winner->iOld1 &&
		ant->iOld2 == winner->iOld2) {
	      pC.push_back(&winner->new1);
	      pC.push_back(&winner->new2);
	      iC = pC.size() - 1;
	      pC.push_back(&winner->new3);
	    } 
	    else {
	      // For ordinary antennae, just add first parton
	      // (except for the antenna which comes immediately after) 
	      if (ant->iOld1 != winner->iOld2) pC.push_back(&ant->old1);  
	      // For the last antenna, also add the antipartner
	      if (iDip == i2) pC.push_back(&ant->old2);	      
	    }
	  }

	  // Subleading color correction
	  int N = pC.size();
	  double sC = m2(pC[iC]->p()+pC[iC-1]->p()+pC[iC+1]->p());
	  double aLC  = sC * antPhys / chargeFactor;
	  double aNLC = sC * 
	    ((showerNLC == 1) ? matchingPtr->Eikonal(pC[iC-1],pC[iC],pC[iC+1])
	     : matchingPtr->A30(pC[iC-1],pC[iC],pC[iC+1]));
	  // Use dimensionless A30 antennae to compute fraction to absorb
	  double fracNLC = matchingPtr->A30(pC[iC-1],pC[iC],pC[iC+1])*sC;
	  // Include this term in sum already
	  double sumLC   = fracNLC;
	  for (int iR=1;iR<=N-2;iR++) {
	    // skip this term, since already included
	    if (iR == iC) continue;
	    sumLC += matchingPtr->A30(pC[iR-1],pC[iR],pC[iR+1])
	      * m2(pC[iR-1]->p()+pC[iR]->p()+pC[iR+1]->p());
	  }
	  fracNLC /= sumLC;
	  double PNLC = 1.0/pow2(NC) * fracNLC * aNLC/aLC;
	  if (1.0 - PNLC < 0.5) {
	    if (PNLC < 1.0) 
	      printOut("VinciaShower","warning: NLC correction > 0.5 * LC");
	    else { 
	      printOut("VinciaShower",
		       "warning: NLC correction > LC one, forced zero.");
	      PNLC = 1.0-TINY;
	    }
	  }

	  // Restore color factor to CA before applying correction
	  Paccept[0]  *= CA/chargeFactor * max(0.5, (1.-PNLC));
          // MR 8.3.2011: variations only if uncertaintyBands
          if (uncertaintyBands) {
            // Variation 10: partially cancel out NLC correction
            Paccept[10] /= max(0.5, (1.0 - PNLC));
            Paccept[10] *= max(0.5, (1.0 - 0.5*PNLC));
            // Variation 11: increase NLC correction
            Paccept[11] /= max(0.5, (1.0 - PNLC));
            Paccept[11] *= max(0.5, (1.0 - 1.5*PNLC));
          }
	}
	
      }
    }

    //------------------------------------------------------------------------
    // *) Optional: post-processing of uncertainty variations

    // Alternative ordering approaches
    if (uncertaintyBands && evolutionMode >= 2 && winner->unOrdering) {
      // Stronger ordering in pT (def is weak ordering in pT)
      double pT2UnOrd = RPtr->Q2E(trial->new1,trial->new2,trial->new3,1);            
      double PimpPT2  = 1.0/(1.0 + pow2(pT2UnOrd/pT2UnOrdOld));
      Paccept[8] *= PimpPT2 / Pimp;
      // mD-ordering
      double mD2UnOrd = RPtr->Q2E(trial->new1,trial->new2,trial->new3,2);            
      double PimpMD   = 1.0/(1.0 + mD2UnOrd/mD2UnOrdOld);
      Paccept[9] *= PimpMD / Pimp;
    }
    
    // Limit uncertainty to factor 2.0 for first branching, to 
    // capture possibly large finite term differences, then level off at 
    // 20% asymptotically, to capture multiple-emission uncertainties.
    if (uncertaintyBands) {
      double limit = 2.0 - min(0.8,0.05*nQCDNow);
      for  (int iVar=1; iVar<nVariations(); iVar++) {
	Paccept[iVar] = min(Paccept[iVar],limit);
	Paccept[iVar] = max(Paccept[iVar],1./limit);
      }      
    }

    //------------------------------------------------------------------------
    // *) Accept or Reject Trial Emission

    // Accept/Reject Trial
    double ran = rndmPtr->flat();
    if (ran > Paccept[0]) {
      if (verbose >= 5) {
	printOut("VinciaShower::branch","=== Branching Vetoed. wPhys/wTrial = "
		 +num2str(Paccept[0]));
      }
      winner->setTrial(false);
      // Check if branching would have happened if we had been using 
      // another radiation function
      if (uncertaintyBands) {
	for  (int iVar=1; iVar<nVariations(); iVar++) {
	  double Pacc = Paccept[0];
	  double Pvar = Paccept[iVar]*Paccept[0];
	  double rw = 1. - (Pvar-Pacc)/(1-Pacc);
	  scaleWeight(rw,iVar);
	}
      }
      return false;    
    } else if (Paccept[0] > 1.0+TINY) {
      if (verbose >= 1) {      
	printOut("VinciaShower::branch",
		 "=== Branching P > 1 at QCD order "+num2str(nQCDNow,1)
		 +", Ant = "+antennaSets[0]->getHumanName(winner->iAnt)
		 +", wPhys/wTrial = "+num2str(Paccept[0])
		 +", Qnew = "+num2str(Qnew));
	if (doMatch && verbose >= 8) cout<<" PME = "<<PME[0]<<endl; 
	if (verbose >= 5) winner->list();
	// Max reweight by factor 10
	scaleWeight(min(10.,Paccept[0]),0);
	if ((Paccept[0] > 1.0 && verbose >= 5) || (Paccept[0] > 2.0 && verbose >= 4) || (Paccept[0] > 10.0 && verbose >= 3)) {
	  cout<<" Momentum Listing: "<<endl;
	  for (int ip=0; ip<int(p.size())-1; ip++) { 		    
	    cout << " p"<<ip<<" = " << p[ip+1];
	  }
	}
      }
      
      // Automatically increase headroom factor when violations encountered,
      // up to a maximum of a factor 10.
      if (nQCDNow <= matchingOrder) {	
	if (nSplitNow == nSplit) { 
	  // Emission
	  if (headRoomFactorEmit[nQCDNow] <= 10.) 
	    headRoomFactorEmit[nQCDNow] *= 1.1;
	} else {
	  // Splitting
	  if (headRoomFactorSplit[nQCDNow] <= 10.) 
	    headRoomFactorSplit[nQCDNow] *= 1.1;
	}
      }
      
    }

    // For accepted branching, rescale uncertainty event weights
    if (uncertaintyBands) {
      for (int iVar=1; iVar<nVariations(); iVar++) {
	scaleWeight(Paccept[iVar],iVar);      	    
      }
    }    

    // Fill histo
    if (HPME != "none" && verbose >= 2) {
      vinciaHistos[HPME].fill(log10(max(TINY,PME[0])));	  	
    }

    //------------------------------------------------------------------------
    // *) Store winner in list of winners

    // Store vector of winners
    winners.push_back(*winner);

    return true;
  }


  //*********

  // Generate the two branching invariants, given Qnew
  // (i.e., generate a zeta, and work out y12, y23 from there)
  
  bool VinciaShower::generateBranching(Branchelemental* trial, double QnewIn) {

    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::generateBranching","begin --------------");
    }
    
    // By default, generate a Z using the Qnew value inside trial
    // but allow user to specify alternative Qnew by hand, if QnewIn > 0
    double QE    = trial->Qnew;
    if (QnewIn > 0.0) QE = QnewIn;
    double Q2E   = pow2(QE);
    double sDip  = trial->s();  
    double mDip  = sqrt(sDip);

    // Retrieve branching Type (emission = 0, splitting > 0)
    int branchFlavor = trial->branchFlavor;  

    
    //****************************************************************************
    // Generate zeta variable, then work out y12, y23
    //****************************************************************************
    
    // Value of evolution variable and phase space boundary
    double yE    = Q2E/sDip;
    double QMin  = getQMin(QE);
    double yMin  = pow2(QMin)/sDip;

    // Z boundaries used for trial generation
    double zMax, zMin;
    double y12, y23;

    // Gluon splitting: trivial z integral
    if (branchFlavor > 0) {
      // Z boundaries
      zMin = 0.0;
      zMax = 1.-yMin;
      // Generate z
      double zNow = rndmPtr->flat() * (zMax-zMin) + zMin;
      // Convert back to branching invariants
      // using Q2E and zeta conventions for evolution in mqq^2
      y12  = zNow;
      y23  = yE;
    }

    // Gluon emission: boundaries depend on evolution variable
    else {

      // For energy ordering, the z window is defined by the hadronization variable
      if (evolutionType == 3) yMin = pow2(cutoffScale)/sDip;
    
      if (evolutionType == 1) {
	// pT-ordering
	zMin = 0.5*(1.0-sqrt(1.0-yMin)); 
	zMax = 0.5*(1.0+sqrt(1.0-yMin)); 
      } else if (evolutionType == 2) {
	// Daughter-mass ordering
	zMin = yMin/2.;
	zMax = 1.-yMin/2.;
      } else if (evolutionType == 3) {
	// For energy ordering the z limits are those imposed by 
	// hadronization in Type 1 variable
	if (cutoffType == 1) {
	  zMin = 0.5*(1.0-sqrt(1.0-yMin));
	  zMax = 0.5*(1.0+sqrt(1.0-yMin));
	} else {
	  zMin = yMin/2.;
	  zMax = 1.-yMin/2.;
	}
      } else if (evolutionType == 4) {
	// Anti-collinear "V" ordering
	zMin  = 0.5*(1.-pow2(1.0-yMin));
	zMax  = 0.5*(1.+pow2(1.0-yMin));
      } else if (evolutionType == 5) {
	// Collinear-safe energy ordering (2*kQ = 1, QE -> pT close to collinear)
	zMin  = 0.5*(1.-pow(1.0-pow2(yMin),0.25));
	zMax  = 0.5*(1.+pow(1.0-pow2(yMin),0.25));
      } 
      // Use same as Type 1 as a catchall (should not occur)
      else {
	zMin = 0.5*(1.0-sqrt(1.0-yMin)); 
	zMax = 0.5*(1.0+sqrt(1.0-yMin)); 
      }

      // Select z
      double rMax = zMax/(1.-zMax);
      double rMin = zMin/(1.-zMin);
      double rNow = rMin * pow(rMax/rMin,rndmPtr->flat());
      double z = rNow/(1.+rNow);  

      // Derive y12, y23 from yE, z
      if (evolutionType == 1) {
	y12 = 0.5*sqrt(yE*rNow);
	y23 = 0.5*sqrt(yE/rNow);
      } 
      else if (evolutionType == 2) {
	if (z < 0.5) {
	  y12 = yE/2.;
	  y23 = y12/rNow;
	} else {
	  y23 = yE/2.;
	  y12 = y23*rNow;
	}
      }
      else if (evolutionType == 3) {
	y12 = z*sqrt(yE);
	y23 = (1.-z)*sqrt(yE);
      } 
      else if (evolutionType == 4) {
	if (z <= 0.5) {
	  y23 = pow2(yE)/2./pow2(z)
	    *( pow2(1.-z)+sqrt(pow2(1.-z)*(1-2*z)) );
	  y12 = y23*z/(1.-z);
	} else {
	  // Re-use solution for z < 0.5 with z <-> 1-z => y12 <-> y23
	  z=1.-z;
	  y12 = pow2(yE)/2./pow2(z)
	    *( pow2(1.-z)+sqrt(pow2(1.-z)*(1-2*z)) );
	  y23 = y12*z/(1.-z);
	} 
      }
      // Use same as Type 1 as a catchall (should not occur)
      else {
	y12 = 0.5*sqrt(yE*rNow);
	y23 = 0.5*sqrt(yE/rNow);
      }
    }

    // Set flavors 
    trial->iAnt         = -1;
    // Flag to set whether the physical antenna is being replaced by a
    // charge conjugate, or permuted, one. In that case, invariants need
    // to be swapped below, before kinematics construction begins.
    bool swapAB     = false;  
    if (branchFlavor == 0) {
      // Gluon emission
      // XY -> XgY type
      trial->new1.id(trial->old1.id());
      trial->new2.id(21);
      trial->new3.id(trial->old2.id());
      if (trial->old1.id() == 21 && trial->old2.id() == 21) {
	// gg->ggg
	trial->iAnt  = iggEmit;
      } else if (trial->old1.id() == 21 || trial->old2.id() == 21) {
	// qg->qgg (and gqbar->ggqbar, but compute for qg->qgg in either case)
	trial->iAnt  = iqgEmit;
	// ... put in massive modification here ...
	// Select between qg->qgg and gqbar->ggqbar
	if (trial->old1.id() == 21) swapAB = true;
      } else {
	// qqbar->qgqbar
	trial->iAnt  = iqqEmit;
	// ... put in massive modification here ...
      }
    } else {
      // Gluon splitting
      if (trial->old1.id() == 21 && trial->old2.id() == 21) {
	// gg->gqbarq (and gg->qbarqg)
	trial->iAnt  = iggSplit;
	// Select between gg->gqbarq and gg->qbarqg
	if (rndmPtr->flat() < 0.5) {
	  trial->new1.id(21);
	  trial->new2.id(-trial->branchFlavor);
	  trial->new3.id(trial->branchFlavor);
	} else {
	  // Single swap = swap. Using gqbarq for qbarqg
	  swapAB     = true;
	  trial->new1.id(-trial->branchFlavor);
	  trial->new2.id(trial->branchFlavor);
	  trial->new3.id(21);
	}
      } else {
	// qg->qqbar'q' (and gqbar->qbar'q'q, but compute for qg->qqbar'q')
	trial->iAnt  = iqgSplit;
	// ... put in massive modification here ...
	// Select between  qg->qqbar'q' and gqbar->qbar'q'q 
	if (trial->old2.id() == 21) {
	  trial->new1.id(trial->old1.id());
	  trial->new2.id(-trial->branchFlavor);
	  trial->new3.id(trial->branchFlavor);
	} else {
	  // Single swap = swap. Using qg for gqbar without swapped invariants in
	  // antPhys.
	  swapAB     = true;
	  trial->new1.id(-trial->branchFlavor);
	  trial->new2.id(trial->branchFlavor);
	  trial->new3.id(trial->old2.id());
	}      
      }
      
    }
    
    // MR 8.7.2010: masses of the particles m0 / mDip (from their ID codes)
    // PS 24.2.2011: check if masses switched on before setting massive
    double mu1(0.), mu2(0.), mu3(0.);
    if ( vinComPtr->isMassive(trial->new1.id()) ) mu1 = trial->new1.m0()/mDip;
    if ( vinComPtr->isMassive(trial->new2.id()) ) mu2 = trial->new2.m0()/mDip;
    if ( vinComPtr->isMassive(trial->new3.id()) ) mu3 = trial->new3.m0()/mDip;
    // Swap 1 and 3 if antenna is swapped
    if ( swapAB ) {
      double tmp = mu1;
      mu1 = mu3;
      mu3 = tmp;
    }

    // For gluon splitting to quarks, translate from sab = (pa+pb)^2 (the
    // evolution variable) sab~ = (pa+pb)^2-ma^2-mb^2 definitions
    if (branchFlavor > 0 && (mu1 > 0.0 || mu2 > 0.0 || mu3 > 0.0) ) {
      y12 += - pow2(mu1) - pow2(mu2);
      y23 += - pow2(mu2) - pow2(mu3);
    }
    
    // Check whether inside physical phase space
    // have to check y12min <= y12 <= y12max first because for 
    // y12min >= y12max a region of positive Gram determinant exists
    double y12min = 2*mu1*mu2;
    double y12max = pow2( 1-mu3 ) - pow2(mu1) - pow2(mu2);
    double y13 = 1. - y12 - y23 - pow2(mu1) - pow2(mu2) - pow2(mu3);
    double gDetDimless = gDet(y12, y23, y13, mu1, mu2, mu3);
    if (gDetDimless <= 0. || y12 >= y12max || y12 <= y12min) {
      if (verbose >= 5) {
		printOut("::generateBranching",
		 "=== Branching Vetoed. Outside physical Phase Space.");      
      }
      return false;
    }

    // Check hadronization veto 1: cutoff scale
    bool hadAccept  = true;    
    if (evolutionType != cutoffType && cutoffType != 0) {
      double Q2forHad = RPtr->yE(y12,y23,cutoffType)*sDip;
      if (Q2forHad < pow2(cutoffScale)) {
	if (verbose >= 5) printOut("::generateBranching",
				   "=== Branching Vetoed. In hadronization region.");            
	hadAccept = false;
      }
    }
    // Check hadronization veto 2: pion mass
    // TODO: massive quarks become massive mesons
    double mMin     = sqrt(RPtr->yE(y12,y23,2)*sDip/2);
    if (mMin < 1.01*particleDataPtr->m0(221) ) {
      if (verbose >= 5) printOut("::generateBranching",
				 "=== Branching Vetoed. m < 1.01*mpi.");            
      hadAccept = false;
    }
    if ( ! hadAccept ) {
      trial->nFailed++;
      return false;
    }

    // Compute physical antenna
    antPhys = getAnt(trial->iAnt)->a2(y12,y23,mu1,mu2,mu3);      
    
    // HYPERJET: special mode for processing events from HEJ 
    // => matching to soft-resummed matrix elements
    // Ignore otherwise
    if (hyperjet && branchFlavor ==0 && trial->system == 0) {
      // If running in hyper-jet mode, subtract soft Eikonal for gluon emission
      // for emissions off the hard interaction
      if (verbose >= 8) 
	printOut("generateBranching","HyperJet mode: Subtracting Eikonal");
      antPhys -= 2*(1.-y12-y23)/(y12*y23);
    }
    
  // Verbose output
  if (verbose >= 8)
    printOut("VinciaShower::generateBranching"," AntPhys  = "
	     +num2str(antPhys/sDip));

  // Get charge factor and kinematics map
  chargeFactor = getAnt(trial->iAnt)->getChargeFactor();
  kineMapType  = getAnt(trial->iAnt)->getKineMapType();
  
  // Reweight trial down to physical antenna by veto
  double wAntPhys  = chargeFactor * antPhys;
  double antTrial, wAntTrial;
  // branchFlavor :
  //     0 : gluon emission
  //   i>0 : gluon splitting to q-qbar pair of flavor i
  if (branchFlavor == 0) {
    antTrial  = 2.0 / y12 / y23;      
    antTrial *= trial->getHeadRoomFactorEmit();
    wAntTrial = antTrial * CA;
  } else {
    antTrial = 1.0 / y23;
    antTrial *= trial->getHeadRoomFactorSplit();
    wAntTrial = antTrial * TR;
  }
  Paccept[0] = wAntPhys/wAntTrial;

  // Verbose output
  if (verbose >= 8) {
    cout<<" AntTrial = "<<setprecision(6)<<antTrial/sDip<<endl;
    cout<<" RAhat    = "<<setprecision(6)<<antPhys/antTrial<<endl;
  }

  // Check if accept <= 0 
  if (Paccept[0] <= 0.0) {
    if (verbose >= 5) printOut("::generateBranching",
	       "=== Branching Vetoed. P(accept) <= 0.0");            
    return false;
  }

  // Compute alternative reweight factors for uncertainty band
  // (ignore if not making uncertainty bands)
  if (uncertaintyBands) {
    for (int iVar=1; iVar<nVariations(); iVar++) {
      // Compute physical antenna for this variation
      int iSet = 0;
      if (iVar >= 1) iSet = 1;
      if (iVar == 4) iSet = 2;
      if (iVar == 5) iSet = 3;
      double antAlt = getAnt(trial->iAnt,iSet)->a2(y12,y23,mu1,mu2,mu3);
      double chargeFacAlt = getAnt(trial->iAnt,iSet)->getChargeFactor();
      // If no NLC corrections, vary all g emission between CF and CA.
      // (If NLC corrections, the variations will be done in the NLC part)
      if (iVar == 10 && branchFlavor == 0 && showerNLC == 0) chargeFacAlt = CA;
      if (iVar == 11 && branchFlavor == 0 && showerNLC == 0) chargeFacAlt = CF;
      double wAntAlt = antAlt * chargeFacAlt;
      Paccept[iVar]  = wAntAlt/wAntPhys;
    }
  }

  // Reweight by alphaS if using pT as argument and not-pT ordering
  double pT2 = 0.25 * RPtr->Q2E(sDip,y12,y23,1);
  if (alphaSmode == 3 && evolutionType != 1) {
    double muRen2 =  max(pow2(alphaSMuMin),pow2(alphaSscaleFactor/0.5) * pT2);
    Paccept[0] *= min(1.0,alphaS.alphaS(muRen2)/alphaSMax);    
  }
  if (uncertaintyBands) {
    double scale2 = sDip;
    if (alphaSmode == 3) scale2 = 4.*pT2;
    if (alphaSmode == 1) scale2 = pow2(trial->Qnew);
    double mu2phys = max(pow2(alphaSMuMin),pow2(alphaSscaleFactor) * scale2);
    double alphaSphys = min(alphaSMax,alphaS.alphaS(mu2phys));
    double alphaVar2  = min(alphaSMax,
          alphaS.alphaS(max(pow2(alphaSMuMin),pT2/uncertaintyBandsKAlphaS)));
    double alphaVar3  = min(alphaSMax,
          alphaS.alphaS(max(pow2(alphaSMuMin),pT2*uncertaintyBandsKAlphaS)));
    Paccept[2] *= alphaVar2/alphaSphys;
    Paccept[3] *= alphaVar3/alphaSphys;
  }

  // Partial NLL matching: impose alphaS(pT) to first order
  // Scale cancellation for uncertainty bands
  // (use alphaS(s) for scale canceling term, to be conservative)
  if (alphaSorder >= 1 && showerNLL >= 1) {
    double muRen2 = sDip;
    if (alphaSmode == 1) muRen2 = RPtr->Q2E(sDip,y12,y23);
    else if (alphaSmode == 3) muRen2 = 4.*pT2;
    muRen2 *= pow2(alphaSscaleFactor);
    muRen2 = max(muRen2,pow2(alphaSMuMin));
    // Use aphaS(sijk) for correction, thus making the correction
    // as small as possible, making the stabilization as conservative
    // as possible.
    double aSCorr   = alphaS.alphaS(sDip);    
    // Target Scale: pT2 for gluon emission, mqq for qqbar
    // (this is before swap, so yqq = y23
    double muRen2Stable = pT2;
    if (branchFlavor != 0) muRen2Stable = y23*sDip;
    // Loop over uncertainty variations
    double DeltaAS       = 0.0;
    double DeltaASpT     = 0.0;
    double NLLFactorUser = 0.0;
    // MR 8.3.2011: corrected to iVar < nVariations()
    for (int iVar=0; iVar < nVariations(); iVar++) {
      // For var >= 4, just copy the one from DEF (which uses pT). 
      if (iVar >= 4) {
	DeltaAS = DeltaASpT;
      }
      else {
	// For all variations, start from pT2
	if (iVar >= 1) muRen2 = pT2;
	// Variations 2 and 3 : Div/Mult by uncertaintyBandsKAlphaS
	if (iVar == 2) muRen2 /= uncertaintyBandsKAlphaS;
	else if (iVar == 3) muRen2 *= uncertaintyBandsKAlphaS;
	double mu = sqrt(muRen2);
	double Nf   = getNf(sqrt(muRen2Stable));
	double NfMu = getNf(mu);
	DeltaAS  = 0.0;
	bool accept = false;
	while (not accept) {
	  double b0  = (11.0*NC - 2.0*NfMu)/(12.0*M_PI);
	  if (NfMu == Nf) {
	    DeltaAS   += aSCorr * b0 * log(muRen2/muRen2Stable);
	    accept = true;
	  } else if (NfMu > Nf) {
	    double Q2Low = pow2(getQMin(NfMu));
	    DeltaAS     += aSCorr * b0 * log(muRen2/Q2Low);      
	    // Update muRen2 and Nf
	    muRen2   = Q2Low;
	    NfMu -= 1;
	  } else if (NfMu < Nf) {
	    double Q2Hi  = pow2(getQMax(NfMu));
	    DeltaAS     += aSCorr * b0 * log(muRen2/Q2Hi);      
	    // Update muRen2 and Nf
	    muRen2   = Q2Hi;
	    NfMu += 1;	
	  }
	}
	// Store the one for alphaS(pT)
	if (iVar == 1) DeltaASpT = DeltaAS;
	double NLLFactor = (1.0 + DeltaAS);
	// Restrict range to 0 < PNLLmu < 2 
	// (expansion otherwise divergent anyway)
	NLLFactor = min(2.0,NLLFactor);
	NLLFactor = max(0.5,NLLFactor);
	if (iVar == 0) {
	  NLLFactorUser = NLLFactor;
	  Paccept[0] *= NLLFactor;
	}
	else {
	  Paccept[iVar] *= NLLFactor/NLLFactorUser;
	}
      }
    }
  }

  //****************************************************************************
  // Generate full kinematics for this trial branching
  //****************************************************************************

  // Swap back if antenna invariants were generated for swapped configuration
  if (swapAB) {
    double tmp = y12;
    y12 = y23;
    y23 = tmp;
    tmp = mu1;
    mu1 = mu3;
    mu3 = tmp;
    if (verbose >= 8) {
      printOut("generateBranching","swapped 1<->3");
    }
  }

  // Verbose output
  if (verbose >= 8) {
    printOut("generateBranching","=> y12 = "+num2str(y12)+
	     "  y23 = "+num2str(y23)+"  y31 = "+num2str(y13)+
	     " m1^2/mDip^2 = "+num2str(pow2(mu1))+
	     " m2^2/mDip^2 = "+num2str(pow2(mu2))+
	     " m3^2/mDip^2 = "+num2str(pow2(mu3)));
  }

  // Compute invariants
  double s12 = y12*sDip;
  double s23 = y23*sDip;
  double s13 = y13*sDip;
  double mass1 = mu1 * mDip;
  double mass2 = mu2 * mDip;
  double mass3 = mu3 * mDip;

  // Can check alternative hadronization vetos here
  if (verbose >= 7) {
    cout<<" (generateBranching:) m  = "<<num2str(sqrt(sDip))
	<<"   m12 ="<<num2str(sqrt(s12))<<"   m23 ="<<num2str(sqrt(s23))
	<<"   m13 ="<<num2str(sqrt(s13))<<"   mass1 ="<<num2str(mass1)
	<<"   mass2 ="<<num2str(mass2)<<"   mass3 ="<<num2str(mass3)<<endl;
    RotBstMatrix M;
    M.toCMframe(trial->old1.p(), trial->old2.p());
    Vec4 p1cm=trial->old1.p();
    Vec4 p2cm=trial->old2.p();  
    p1cm.rotbst(M);
    p2cm.rotbst(M);
    cout <<" (generateBranching:) starting dipole in CM\n";
    cout <<" p1cm = "<<p1cm<<" p2cm = "<<p2cm<<endl;
  }

  // Set up kinematics in rest frame.
  // MR 7.7.2010: use mass1, mass2, mass3, rewrite the determination
  double E1 = 1/mDip*(pow2(mass1) + s12/2 + s13/2);
  double E2 = 1/mDip*(s12/2 + pow2(mass2) + s23/2);
  double E3 = 1/mDip*(s13/2 + s23/2 + pow2(mass3));
  double ap1 = sqrt( pow2(E1) - pow2(mass1) );
  double ap2 = sqrt( pow2(E2) - pow2(mass2) );
  double ap3 = sqrt( pow2(E3) - pow2(mass3) );
  double cos12 = (E1*E2 - s12/2)/(ap1*ap2);
  // use positive square root for sine
  double sin12 = sqrt(abs(1.0 - pow2(cos12)));
  double cos13 = (E1*E3 - s13/2)/(ap1*ap3);
  double sin13 = sqrt(abs(1.0 - pow2(cos13)));
  // make sure that cosine and sine are both between zero and one
  if (verbose >= 3) {
    if (!(-1 <= cos12 && cos12 <= 1 && -1 <= sin12 && sin12 <= 1 
	  && -1 <= cos13 && cos13 <= 1 && -1 <= sin13 && sin13 <= 1)) {
      printOut("generateKinematics",
	       "inconsistent invariants, values are\nsDip = "
	       + num2str(sDip) + ",\ns12 = "
	       + num2str(s12) + ",\ns13 = " + num2str(s13)
	       + ",\ns23 = " + num2str(s23) + ",\nmass1 = "
	       + num2str(mass1) + ",\nmass2 = " + num2str(mass2)
	       + ",\nmass3 = " + num2str(mass3) + ",\nE1 = "
	       + num2str(E1) + ",\nE2 = " + num2str(E2)
	       + ",\nE3 = " + num2str(E3) + ",\nap1 = " + num2str(ap1)
	       + ",\nap2 = " + num2str(ap2) + ",\nap3 = "
	       + num2str(ap3) + ",\ngdetdimless = "
	       + num2str(gDetDimless) 
	       );
    }
  }
  // If massless, check kinematics for massless case
  if (verbose >= 3) {
    double sm1Chk = (vinComPtr->isMassive(trial->new1.id()) ?
		     pow2(trial->new1.m0()) : 0.0); 
    double sm2Chk = (vinComPtr->isMassive(trial->new2.id()) ? 
		     pow2(trial->new2.m0()) : 0.0);
    double sm3Chk = (vinComPtr->isMassive(trial->new3.id()) ? 
		     pow2(trial->new3.m0()) : 0.0);
    double s12Chk = s12 + sm1Chk + sm2Chk;
    double s13Chk = s13 + sm1Chk + sm3Chk;
    double s23Chk = s23 + sm2Chk + sm3Chk;
    double xChk=2.0*sqrt(sDip);
    double E1Chk=(sDip-s23Chk+sm1Chk)/xChk;
    double E2Chk=(sDip-s13Chk+sm2Chk)/xChk;
    double E3Chk=(sDip-s12Chk+sm3Chk)/xChk;
    double ap1Chk=sqrt(pow2(E1Chk)-sm1Chk);
    double ap2Chk=sqrt(pow2(E2Chk)-sm2Chk);
    double ap3Chk=sqrt(pow2(E3Chk)-sm3Chk);
    double cos12Chk=(2*E1Chk*E2Chk+sm1Chk+sm2Chk-s12Chk)/2.0/ap1Chk/ap2Chk;
    double sin12Chk=sqrt(abs(1.0-cos12Chk*cos12Chk));
    double cos13Chk=(2*E1Chk*E3Chk+sm1Chk+sm3Chk-s13Chk)/2.0/ap1Chk/ap3Chk;
    double sin13Chk=sqrt(abs(1.0-cos13Chk*cos13Chk));
    if (abs(sin12-sin12Chk) > TINY || abs(sin13-sin13Chk) > TINY || abs(ap3-ap3Chk) 
	> TINY || abs(ap2-ap2Chk) > TINY || abs(ap1-ap1Chk) > TINY) {
      printOut("generateBranching",string("Error! consistency check failed.")
	       + string(" Old != New "));
      if (verbose >= 3) {
	trial->list();
	cout<<" New : |p| = " <<ap1<<" "<<ap2<<" "<<ap3<<" sin12 = "<<sin12<<" sin13 = "<<sin13<<endl; 
	cout<<" Old : |p| = " <<ap1Chk<<" "<<ap2Chk<<" "<<ap3Chk<<" sin12 = "<<sin12Chk<<" sin13 = "<<sin13Chk<<endl; 
	cout<<" Flavors = "<<trial->new1.id()<<" "<<trial->new2.id()<<" "
	    <<trial->new3.id()<<" masses = "<<mass1<<" "<<mass2<<" "<<mass3
	    <<" vs mChk = "<<sqrt(sm1Chk)<<" "<<sqrt(sm2Chk)<<" "<<sqrt(sm3Chk)
	    <<endl;
      }
    } else {
      if (verbose>=6) {
	printOut("generateBranching",string("OK: consistency check passed.")
		 + string(" Old = New "));
      }
      

    }
  }
  
  // Set momenta in CMz frame
  // (= frame with 1 oriented along positive z axis and event in (x,z) plane)
  Vec4 p1(0.0,0.0,ap1,E1);
  Vec4 p2(-ap2*sin12,0.0,ap2*cos12,E2);
  Vec4 p3(ap3*sin13,0.0,ap3*cos13,E3);

  // Verbose output
  if (verbose >= 8) {
    cout <<" (generateBranching:) configuration in CM* (def: 1 along z)\n";
    cout <<" k1* =  "<<p1<<" k2* =  "<<p2<<" k3* =  "<<p3<<endl;
  }  

  // Choose azimuthal (phi) angle around dipole axis
  double phi = 2*M_PI*rndmPtr->flat();

  // Choose Global rotation around axis perpendicular to event plane
  double psi;

  // MR 7.7.2010: add if clause for masses, complain and return false if
  // kineMapType is 1 or 2
  // MR 7.7.2010: technically I should check for massive parents
  // as well, but since in QCD massive parents do always have
  // massive daughters, I omitted it
  // Types 1 and 2 only implemented for massless particles
  if (mu1 >= TINY || mu2 >= TINY || mu3 >= TINY) {
    // Massive particles
    // MR 7.7.2010: massive generalization of the Kosower map as in the notes
    // the formula is unstable for s13 close to its minimum, maybe additional
    // checks are needed
    // I and K  denote the parents
    // MR 7.7.2010:
    // most of the variables below are technically unnecessary, it might
    // be preferrable to use something like inline, don't bother for now
    double gdet = pow3(sDip)*gDetDimless;
    double mI = trial->old1.m();
    double mK = trial->old2.m();
    double sigmasqua = sDip + pow2(mI) - pow2(mK);
    double sIK = sDip - pow2(mI) - pow2(mK);
    double sIKmin = 2*mI*mK;
    double s12min = 2*mass1*mass2;
    double s23min = 2*mass2*mass3;
    double s13min = 2*mass1*mass3;
    double rantmap = 
      (
       sigmasqua+
       sqrt( pow2(sIK) - pow2(sIKmin) )*
       ((s23-s23min)-(s12-s12min))/(s12-s12min + s23-s23min)
       )/(2*sDip);
    double bigRantmap =
      sqrt(
	   16*gdet*
	   (sDip*rantmap*(1-rantmap) - (1 - rantmap)*pow2(mI)
	    - rantmap*pow2(mK))
	   +(pow2(s13) - pow2(s13min))*(pow2(sIK) - pow2(sIKmin))
	   );
    double p1dotpI =
      (sigmasqua*(pow2(s13) - pow2(s13min))*
       (sDip + pow2(mass1) - pow2(mass2) - pow2(mass3) - s23)
       +8*rantmap*(sDip + pow2(mass1) - pow2(mass2) - pow2(mass3) - s23)*gdet
       -bigRantmap*(pow2(s13) - pow2(s13min) + s12*s13-2*s23*pow2(mass1)))
      /(4*(4*gdet+sDip*(pow2(s13) - pow2(s13min))));
    // need the norm of the three-momentum and the energy of the first parent
    // particle
    double apI =
      sqrt(
	   (pow2(sDip) + pow4(mI) + pow4(mK) - 2*sDip*pow2(mI)
	    - 2*sDip*pow2(mK) -2*pow2(mI)*pow2(mK)) 
	   )/(2*mDip);
    double EI = sqrt( pow2(apI) + pow2(mI) );
    double cospsi = ((E1*EI) - p1dotpI)/(ap1*apI);
    psi = acos( cospsi );    
  } else {
    if (kineMapType == 1) {
      // ARIADNE map. 
      psi = E3*E3/(E1*E1+E3*E3)*(M_PI-acos(cos13));
    } else if (kineMapType == 2) {
      // psi PYTHIA-like. "Recoiler" remains along z-axis
      psi = 0.;
      if (s12 < s23 || (s12 == s23 && rndmPtr->flat() > 0.5) ) psi=M_PI-acos(cos13);
    } else if (kineMapType == 3) {
      // Kosower's map. Similar to ARIADNE. 
      double f=y23/(y12+y23);
      double rho=sqrt(1.0+4.0*f*(1.0-f)*y12*y23/y13);
      double yaa=-((1.0-rho)*y13+2.0*f*y12*y23)/2.0/(1.0-y12);
      // Bug corrected for v 1.019
      //    psi=1.0+2.0*yaa/(1.0-y23);    
      psi=acos(1.0+2.0*yaa/(1.0-y23));    
    }
    // MR 11.7.2010: implement massive dipole mapping, should
    // be type 2 but I write it down as type 4 to separate it
    else if ( kineMapType == 4 ) {
      if (verbose>=7) {
	printOut("generateKinematics", "using map 4");
      }
      // parton 3 is the default recoiler
      psi = M_PI - acos(cos13);
      if ( !(s12 < s23) ) {
	if (verbose>=8) {
	printOut("generateKinematics",
		 "parton 1 is chosen as the recoiler");
	}
	psi = 0;
      }
    /*
      the block commented out down here is the massive dipole
      mapping written in analogy to the antenna mapping
      it is superfluous at this point because we can just
      use the angle in the CM* frame, but I kept it here for
      reference
    */
    /*
      double gdet = pow3(sDip)*gdetdimless;
      double mI = trial->old1.m();
      double mK = trial->old2.m();
      double sigmasqua = sDip + pow2(mI) - pow2(mK);
      double sIK = sDip - pow2(mI) - pow2(mK);
      double sIKmin = 2*mI*mK;
      double s13min = 2*mass1*mass3;
      double kaellen = pow2(sDip-(pow2(mass1)+pow2(mass2)+s12)-pow2(mass3) )
      -4*(pow2(mass1) + pow2(mass2) + s12)*pow2(mass3);
      double rantmap = 
      (
      sigmasqua+
      sqrt( pow2(sIK) - pow2(sIKmin) )*
      (sDip - pow2(mass1) - pow2(mass2) + pow2(mass3) - s12)
      /sqrt( kaellen )
      )/(2*sDip);
      double bigRantmap =
      sqrt(
      16*gdet*
      (sDip*rantmap*(1-rantmap) - (1 - rantmap)*pow2(mI)
      - rantmap*pow2(mK))
      +(pow2(s13) - pow2(s13min))*(pow2(sIK) - pow2(sIKmin))
      );
      double p1dotpI =
      (sigmasqua*(pow2(s13) - pow2(s13min))*
      (sDip + pow2(mass1) - pow2(mass2) - pow2(mass3) - s23)
      +8*rantmap*(sDip + pow2(mass1) - pow2(mass2) - pow2(mass3) - s23)*gdet
      -bigRantmap*(pow2(s13) - pow2(s13min) + s12*s13-2*s23*pow2(mass1)))
      /(4*(4*gdet+sDip*(pow2(s13) - pow2(s13min))));
      // need the norm of the three-momentum and the energy of the first parent
      // particle
      double apI =
      sqrt(
      (pow2(sDip) + pow4(mI) + pow4(mK) - 2*sDip*pow2(mI)
      - 2*sDip*pow2(mK) -2*pow2(mI)*pow2(mK)) 
      )/(2*mDip);
      double EI = sqrt( pow2(apI) + pow2(mI) );
      double cospsi = ((E1*EI) - p1dotpI)/(ap1*apI);
      psi = acos( cospsi );
      // check that this is nothing but M_PI - cos13 in the CM* frame
      printOut("generateKinematics", "abs(psi -( pi - acos(cos13))) is "
      + num2str( abs( psi - (M_PI - acos(cos13)))) );
    */
    }
    else {
      // if the kineMapType is not recognized, complain and return
      // false
      if (verbose >= 3) {
	printOut("generateKinematics", "kineMapType "
		 + num2str( kineMapType )
		 + " not recognized, returning false ");
      }
      return false;
    }
  }
  
  // Perform global rotations
  p1.rot(psi,phi);
  p2.rot(psi,phi);
  p3.rot(psi,phi);

  // verbose output
  if (verbose >= 7) {
    cout <<" (generateBranching:) phi = "<<phi<<" psi = "<<psi<<endl;
    cout <<" (generateBranching:) final momenta in CM\n";
    cout <<" k1cm = "<<p1<<" k2cm = "<<p2<<" k3cm = "<<p3<<endl;
  }  

  // Rotate and boost to LAB frame
  RotBstMatrix M;
  M.fromCMframe(trial->old1.p(),trial->old2.p());
  if (verbose >= 8) {
    cout <<" (generateBranching:) boosting to LAB frame defined by\n";
    cout <<" p1 =   "<<trial->old1.p()<<" p2 =   "<<trial->old2.p()<<endl;}

  p1.rotbst(M);
  p2.rotbst(M);
  p3.rotbst(M);
  if (verbose >= 7) {
    cout <<" (generateBranching:) final momenta in LAB\n";
    cout <<" k1 =   "<<p1<<" k2 =   "<<p2<<" k3 =   "<<p3<<endl;}

  // Save momenta
  trial->new1.p(p1);
  trial->new2.p(p2);
  trial->new3.p(p3);

  // Verbose output: check momentum mapping by explicit reclustering
  // --> MASSIVE MODIFICATION <--
  if (verbose >= 3) {
    vector<Vec4> pNew, pClu;
    pNew.push_back(trial->new1.p());
    pNew.push_back(trial->new2.p());
    pNew.push_back(trial->new3.p());
    // MR 7.7.2010: in the massive case, the map needs to know the parent
    // masses, do a variant vc3to2massive which is called for massive
    // daughter particles (again, technically I should check for massive
    // parents as well)
    bool clusterok = false;
    if (mu1 >= TINY || mu2 >= TINY || mu3 >= TINY) {
      double mI = (vinComPtr->isMassive(trial->old1.id()) ? trial->old1.m() : 0.0);
      double mK = (vinComPtr->isMassive(trial->old2.id()) ? trial->old2.m() : 0.0);
      // Massive case only implemented for types >= 3
      int kineMapTemp = max(3,kineMapType);
      clusterok = vinCluPtr->vc3to2massive(kineMapTemp,pNew,0,1,2,mI,mK,pClu);
      if (!clusterok && verbose >= 3)
	printOut("VinciaShower::generateKinematics",
		 "vc3to2massive returned false");
    }
    else {
      clusterok = vinCluPtr->vc3to2(kineMapType,pNew,0,1,2,pClu);
      if (!clusterok && verbose >= 3)
	printOut("VinciaShower::generateKinematics",
		 "vc3to2 returned false");
    }
    // Check (e,p) conservation
    // If particles have been forced massless, only check CM energy cons 
    if ( (trial->old1.m2() > TINY && ! vinComPtr->isMassive(trial->old1.id())) ||
	 (trial->old2.m2() > TINY && ! vinComPtr->isMassive(trial->old2.id())) ) {
    } else {
      // PS: changed to dimensionless comparison 
      double DeltaEP = (pow2(pClu[0].px()-trial->old1.px())
			+ pow2(pClu[0].py()-trial->old1.py())
			+ pow2(pClu[0].pz()-trial->old1.pz())
			+ pow2(pClu[0].e() -trial->old1.e())
			+ pow2(pClu[1].px()-trial->old2.px())
			+ pow2(pClu[1].py()-trial->old2.py())
			+ pow2(pClu[1].pz()-trial->old2.pz())
			+ pow2(pClu[1].e() -trial->old2.e()));
      DeltaEP /= sDip;
      if (!clusterok || abs(DeltaEP) > TINY || isnan(DeltaEP) ) {
	// MR 20.7.2010: since the on-shell condition for the reclustered
	// momenta seems to be the most sensitive quantity, have a 
	// reduced message if vc3to2 returned false but the reclustering was
	// within the limits
	if (!clusterok && abs(DeltaEP) < TINY && ! isnan(DeltaEP) ) {
	  printOut("VinciaShower::generateKinematics", "   ... but "
		   + string("reclustering test passed.") );
	  // print all the detail if verbose >= 4
	  if (verbose >= 4) {
	    cout << "p1    = " << trial->new1.p();
	    cout << "p2    = " << trial->new2.p();
	    cout << "p3    = " << trial->new3.p();
	    cout << "pahat = " << pClu[0];
	    cout << "pbhat = " << pClu[1];
	    cout << "pa    = " << trial->old1.p();
	    cout << "pb    = " << trial->old2.p();
	    cout << setprecision(16);
	    cout << "TINY		= " << TINY							<< endl;
	    cout << "m(p1)		= " << trial->new1.p().mCalc() << endl;
	    cout << "m(p2)		= " << trial->new2.p().mCalc() << endl;
	    cout << "m(p3)		= " << trial->new3.p().mCalc() << endl;
	    cout << "m(pa)		= " << trial->old1.p().mCalc() << endl;
	    cout << "m(pb)		= " << trial->old2.p().mCalc() << endl;
	    cout << "m(pahat)   = " << pClu[0].mCalc() << endl;
	    cout << "m(pbhat)   = " << pClu[1].mCalc() << endl;
	    cout << "DeltaEP = " << DeltaEP << endl;
	    cout << "Ecm	 = " << mDip << endl;
	    cout << "s12	 = " << s12 << endl;
	    cout << "s23	 = " << s23 << endl;
	    cout << "psi	 = " << psi << endl;
	    cout << "phi	 = " << phi << endl;
	  }
	}
	else {
	  printOut("generateKinematics", "reclustering failed");
	  // MR 16.7.2010: if the reclustering fails, I want to have
	  // a lot of information about the event
	  cout << "p1    = " << trial->new1.p();
	  cout << "p2    = " << trial->new2.p();
	  cout << "p3    = " << trial->new3.p();
	  cout << "pahat = " << pClu[0];
	  cout << "pbhat = " << pClu[1];
	  cout << "pa    = " << trial->old1.p();
	  cout << "pb    = " << trial->old2.p();
	  cout << setprecision(16);
	  cout << "TINY		= " << TINY							<< endl;
	  cout << "m(p1)		= " << trial->new1.p().mCalc() << endl;
	  cout << "m(p2)		= " << trial->new2.p().mCalc() << endl;
	  cout << "m(p3)		= " << trial->new3.p().mCalc() << endl;
	  cout << "m(pa)		= " << trial->old1.p().mCalc() << endl;
	  cout << "m(pb)		= " << trial->old2.p().mCalc() << endl;
	  cout << "m(pahat)   = " << pClu[0].mCalc() << endl;
	  cout << "m(pbhat)   = " << pClu[1].mCalc() << endl;
	  cout << "DeltaEP = " << DeltaEP << endl;
	  cout << "Ecm	 = " << mDip << endl;
	  cout << "s12	 = " << s12 << endl;
	  cout << "s23	 = " << s23 << endl;
	  cout << "psi	 = " << psi << endl;
	  cout << "phi	 = " << phi << endl;
	}
      }
      /*
	double DeltaEP = sqrt(pow2(pClu[0].px()-trial->old1.px())
	+ pow2(pClu[0].py()-trial->old1.py())
			  + pow2(pClu[0].pz()-trial->old1.pz())
			  + pow2(pClu[0].e()-trial->old1.e())
			  + pow2(pClu[1].px()-trial->old2.px())
			  + pow2(pClu[1].py()-trial->old2.py())
			  + pow2(pClu[1].pz()-trial->old2.pz())
			  + pow2(pClu[1].e()-trial->old2.e()));
    DeltaEP /= pClu[0].e()+pClu[1].e();
    if (DeltaEP > TINY) {
      printErr("generateKinematics","relative deviation = "+num2str(DeltaEP)
	       +" in reclustering 3->2 kinematics");
	  // MR 16.7.2010: want to have more information if the reclustering fails
	  cout << "pi    = " << trial->new1.p();
	  cout << "pj    = " << trial->new2.p();
	  cout << "pk    = " << trial->new3.p();
	  cout << "pahat = " << pClu[0];
	  cout << "pbhat = " << pClu[1];
	  cout << "pa    = " << trial->old1.p();
	  cout << "pb    = " << trial->old2.p();
	  cout << setprecision(10);
	  cout << "Ecm	 = " << mDip << endl;
	  cout << "s12	 = " << s12 << endl;
	  cout << "s23	 = " << s23 << endl;
	  cout << "psi	 = " << psi << endl;
	  cout << "phi	 = " << phi << endl;
	}
	// MR 11.7.2010: added test whether DeltaEP is a real, I had the
	// creation of nan slip past the clause above
	if (isnan(DeltaEP)) {
		printErr("generateKinematics", "DeltaEP is NaN, "
					+ string("probably a vector with nan was ")
					+ string("returned by the 3->2 clustering") );
	}
    */
      if (verbose >= 7) {
	cout<<" Checking reclustering to "<<trial->iOld1<<"-"
	    <<trial->iOld2<<" using map "<<kineMapType<<endl;
	cout<<"      pahat = "<<num2str(pClu[0].px())<<" "<<num2str(pClu[0].py())
	    <<" "<<num2str(pClu[0].pz())<<" "<<num2str(pClu[0].e());
	cout<<"      pbhat = "<<num2str(pClu[1].px())<<" "<<num2str(pClu[1].py())
	    <<" "<<num2str(pClu[1].pz())<<" "<<num2str(pClu[1].e())<<endl;
	cout<<"      "<<setw(5)<<trial->iOld1<<" = "<<num2str(trial->old1.px())
	    <<" "<<num2str(trial->old1.py())
	    <<" "<<num2str(trial->old1.pz())
	    <<" "<<num2str(trial->old1.e());
	cout<<"      "<<setw(5)<<trial->iOld2<<" = "<<num2str(trial->old2.px())
	    <<" "<<num2str(trial->old2.py())
	    <<" "<<num2str(trial->old2.pz())
	    <<" "<<num2str(trial->old2.e())<<endl;
      }
    }
  }
  
  return true;
}

//*********

// Return charge * a2() 
//    for a specific antenna = iAnt
//    with partons stored in p, indices a, r, and b.

double VinciaShower::a2(int iAnt, vector<Vec4> p, int a, int r, int b, 
			double charge) {

  // Verbose output
  if (verbose >= 7) {
    printOut("VinciaShower::a2","begin --------------");
  }

  if (iAnt < 0 ) return 0.0;
  // MR 9.7.2010: inserted masses
  double sar  = 2*p[a]*p[r];
  double srb  = 2*p[r]*p[b];
  double massa = p[a].mCalc();
  double massr = p[r].mCalc();
  double massb = p[b].mCalc();	  
  double sDip    = m2(p[a],p[r],p[b]);
  if (sDip == 0.0) return 0.0;
  return charge * getAnt(iAnt)->a2ofS(sar, srb, sDip, massa, massr, massb);  

}

// Return a2() for default charge, otherwise as above

double VinciaShower::a2(int iAnt,vector<Vec4> p, int a,int r,int b) {
  if (iAnt < 0 ) return 0.0;
  double charge = getAnt(iAnt)->getChargeFactor();
  return a2(iAnt,p,a,r,b,charge);
}

//*********

/* Check whether radiation falls into sector
Important Note: not yet operational for gluon loops! (e.g., H->gg)
Inputs: 
   event : event record before branching
   trial : proposed trial branching
Output:
   sectorAccept : false if branching should be vetoed
*/

bool VinciaShower::sectorAccept(Event& event,Branchelemental* trial) {

  // Only do check if evolution mode is not global
  if ( ! sectorOrdering ) return true;

  // Verbose output
  if (verbose >= 7) {
    printOut("VinciaShower::sectorAccept","begin --------------");
  }

  // Check if this sector has smallest pT
  double pTSector = sqrt(RPtr->Q2E(trial->new1,trial->new2,trial->new3,1)/4.);
  double pTMin = mSystem/2.;
  for (int iDip = 0; iDip < int(branchelementals.size()); iDip++) {
    int ia = branchelementals[iDip].iOld1;
    int ir = branchelementals[iDip].iOld2;
    int ib = branchelementals[iDip].iR;
    // skip if no right-hand neighbor
    if (ib == 0) continue;
    // Compute pT for this triplet
    Vec4 pa = event[ia].p();
    Vec4 pr = event[ir].p();
    Vec4 pb = event[ib].p();
    // use new particles if neigboring pts affected by recoils
    if (ir == trial->iOld1) {
      pr = trial->new1.p();
      pb = trial->new2.p();
    } else if (ib == trial->iOld1) {
      pb = trial->new1.p();
    }
    if (ia == trial->iOld2) {
      pa = trial->new3.p();
    } else if (ir == trial->iOld2) {
      pa = trial->new2.p();
      pr = trial->new3.p();
    }
    // Keep track of smallest pT so far
    pTMin = min(pTMin,sqrt(RPtr->Q2E(pa,pr,pb,1)/4.));
    if (verbose >= 8) {
      printOut("sectorAccept","testing sector no "+num2str(iDip)+" pTMin = "
	       +num2str(pTMin)+" (pTsector = "+num2str(pTSector)+")");
    }
    if (pTSector > pTMin) return false;
  }

  // Return 
  return true;
  
}

//********* 

// Set event weight, check for non-unity and negative weights

  bool VinciaShower::setWeight(double weightIn, int iWeightIn) {

    // Verbose output
    if (verbose >= 7) {
      printOut("VinciaShower::setWeight","begin --------------");
    }

    if (! existsWeight(iWeightIn) ) return false;
    
    // For default / USR weight set, check for nonunity event weights.
    if ( iWeightIn == 0 ) {
      if (abs(abs(weightIn) - 1.0) > TINY && 
	  abs(abs(weightSave[iWeightIn])-1.0) < TINY) {
	nonunityWeightOccurred = true;     
	if (nNonunityWeight <= 5.0) 
	  if (verbose >= 1) 
	    printOut("VinciaShower::setWeight","Nonunity weight occurred, w = "+
		     num2str(weightIn)+((abs(nNonunityWeight-5.0) < TINY) ? 
					": further warnings suppressed" : " "));
      }
      
      // Check for negative weights 
      if (weightSave[iWeightIn] > 0.0 && weightIn < 0.0) {
	negativeWeightOccurred = true;
	if (nNegativeWeight <= 5)
	  if (verbose >= 1) 
	    printOut("VinciaShower::setWeight","Negative weight occurred, w = "+
		     num2str(weightIn)+((nNegativeWeight == 5) ? 
					": further warnings suppressed" : " "));
      } 

    }

    // Save new weight
    weightSave[iWeightIn] = weightIn;

    // Everything ok
    return true;
    
  }
  

//********* 

// OUTPUT ROUTINES

// Print header information
void VinciaShower::header() {

  // Verbose output
  if (verbose >= 7) {
    printOut("VinciaShower::header","begin --------------");
  }

  cout<<setprecision(3);
  cout.setf(ios::left);
  cout << "\n";
  cout << " *-------  VINCIA"<<fixed<<setw(6)<<version
       <<" Global Initialization  ---------------------------------------*\n";
  cout << " | \n";
  cout << " | Vincia                             = "<<setw(12)<<
    ( (settingsPtr->flag("Vincia") ? "on" : "off")) <<"\n |\n";
  if (settingsPtr->flag("Vincia")) {
    cout << " | VinciaMatching:                    = ";
    if (matching && matchingOrder >= 1) {
      cout <<setw(12)<< "on"<<"\n"; 
      cout <<" |                 order              = "<<setw(12)<<matchingOrder<<endl;
      cout <<" |                 fullColor          = "<<setw(12)
	   <<(matchingFullColor ? "on":"off")<<"\n";
      cout <<" |                 MGInterface        = "<<setw(12)
	   <<(isMGOn ? "on":"off")<<"\n";
      if (verbose >= 2) {
	cout <<" |                 regOrder           = "<<setw(12)<<matchingRegOrder<<endl;
	cout <<" |                 regShape           = "<<setw(12)<<matchingRegShape<<endl;
	cout <<" |                 regType            = "<<setw(12)<<matchingScaleType<<endl;
      }
      cout <<" |                 regScaleIsAbs      = "<<setw(12)
	   <<(matchingScaleIsAbs ? "on":"off")<<"\n";
      if (matchingScaleIsAbs) 
	cout <<" |                 regScale           = "<<setw(12)<<matchingScale<<endl;
      else 
	cout <<" |                 regScaleRatio      = "<<setw(12)<<matchingScaleRatio<<endl;
    }    
    else {
      cout<<setw(12)<<"off"<<"\n";
    }
    cout << " |\n";
    cout << " | VinciaShower:   evolutionType      = "<<setw(12)
	 <<settingsPtr->mode("VinciaShower:evolutionType")<<"\n";
    cout << " |                 evolutionMode      = "<<setw(12)<<
      settingsPtr->mode("VinciaShower:evolutionMode")<<"\n";
    cout << " |                 nGluonToQuark      = "
	 << setw(12)<<nGluonToQuark<<"\n";
    if (verbose >= 2) {
      cout << " |                 isMassiveS         = "
	   << setw(12)<<(vinComPtr->isMassive(3) ? "yes":"no")<<"\n";
      cout << " |                 isMassiveC         = "
	   << setw(12)<<(vinComPtr->isMassive(4) ? "yes":"no")<<"\n";
      cout << " |                 isMassiveB         = "
	   << setw(12)<<(vinComPtr->isMassive(5) ? "yes":"no")<<"\n";
    }
    cout << " |                 QMinAll            = "
	 << setw(12)<<QMinAll<<"\n";
    if (sectorOrdering) {
      cout << " |                 sectorOrdering     = "<<setw(12)<<
	( settingsPtr->flag("VinciaShower:sectorOrdering") ? "on":"off")<<"\n";
    }
    cout << " |                 NLC level          = "
	 <<setw(12)<<settingsPtr->mode("VinciaShower:NLC")<<"\n"
	 << " |                 NLL muR            = "
	 <<setw(12)<<(( settingsPtr->mode("VinciaShower:NLL") >= 1) ? "on":"off")<<"\n";
    cout << " |\n"
	 << " | Vincia:         cutoffType         = "
	 << setw(12)<<settingsPtr->mode("Vincia:cutoffType")<<(settingsPtr->mode("Vincia:cutoffType") == -1 ? " (Pythia 8 pTevol)" :" ")<<"\n"    
	 << " |                 cutoffScale        = "
	 <<setw(12)<<scientific<<cutoffScale<<"\n |\n"
	 << " | Alpha_s:        alphaS(mZ)         = "
	 <<setw(12)<<scientific<<alphaSvalue<<"\n"
	 << " |                 order              = "
	 <<setw(12)<<alphaSorder<<"\n";
    if (alphaSorder >= 1) {
      cout << " |                 mode               = "
	   <<setw(12)<<settingsPtr->mode("Vincia:alphaSmode")<<"\n"
	   << " |                 scaleFactor        = "
	   <<setw(12)<<scientific<<alphaSscaleFactor<<"\n"
	   << " |                 alphaSMax          = "
	   <<setw(12)<<num2str(alphaSMax)<<"\n |\n";
    }

    if ( settingsPtr->word("VinciaShower:antennaFile") != "none") {
      cout<<" | Antennae:       antennaFile        = "
	  <<settingsPtr->word("VinciaShower:antennaFile")<<"\n |\n"; 	
    }
    
    if (verbose >= 2) {
      for (unsigned int i=0;i<=4;i++) {
	cout.setf(ios::left);
	cout<<setprecision(2);
	cout<<" | * "<<getAnt(i)->getHumanName()
	    <<" ------------------------------------------------------------------------\n";
	if (getAnt(i)->isOn()) { 
	  cout<<" |   chargeFactor       = "<<fixed
	      <<setw(6)<<getAnt(i)->getChargeFactor()<<"\n";
	  cout<<" |   useFiniteTerms     = "<<setw(6)<<
	    ( settingsPtr->flag("VinciaShower:useFiniteTerms") ? "on":"off")<<"\n";
	  cout<<" |   kineMapType        = "
	      <<setw(6)<<getAnt(i)->getKineMapType()<<"\n";
	  if (getAnt(i)->getInputMode() == 1) { 
	    int iprc = 2;
	    cout.setf(ios::right);
	    cout<<" |   Ant(ya,yb)         =  ";
	    bool linechange=false;
	    bool hasWritten=false;
	    for (int a=-1; a <= 3; a++) {
	      // Separate lines for singular and non-singular terms
	      if (a == 0) linechange = true;
	      double Caa = getAnt(i)->getC(a,a);
	      if (Caa != 0.0) {
		if (linechange) cout<<endl<<" |                         ";
		linechange=false;
		cout<<" ";
		if (Caa < 0.0) cout<<"- "; 
		else if (a >= 0) cout<<"+ ";
		iprc = 2;
		if ( int(abs(Caa) * 100 + 0.5) % 10 == 0) iprc = 1;
		if (iprc == 1 && int(abs(Caa) * 10 +0.5) % 10 == 0) iprc=0;
		if (a <= 0 || abs(Caa) != 1.0) cout<<setprecision(iprc)<<abs(Caa);
		if (a == -1) cout <<"/ya/yb";
		else if (a == 1) cout <<(abs(Caa)!=1.0?"*":"")<<"ya*yb";
		else if (a >= 2) cout <<(abs(Caa)!=1.0?"*":"")<<"ya^"<<a<<"*yb^"<<a;
		hasWritten=true;
	      }
	      for (int b=a+1; b <= 3; b++) {
		double Cab = getAnt(i)->getC(a,b) ;
		if (Cab != 0.0) {
		  if (linechange) cout<<endl<<" |                         ";
		  linechange=false;
		  cout<<" ";
		  if (Cab > 0.0 && hasWritten) cout<<"+ ";
		  else if (Cab < 0.0) cout<<"- ";
		  iprc=2;
		  if ( int(abs(Cab) * 100 + 0.5) % 10 == 0) iprc = 1;
		  if (iprc == 1 && int(abs(Cab) * 10 +0.5) % 10 == 0) iprc=0;
		  if (a == -1 && b == 0) 
		    cout<<setprecision(iprc)<<abs(Cab);
		  else if (abs(Cab) != 1.0) 
		    cout<<setprecision(iprc)<<abs(Cab)<<"*";
		  if (a == 1) cout <<"ya";
		  else if (a >= 2) cout <<"ya^"<<a;
		  if (b == 1) cout <<(a>=1?"*":"")<<"yb";
		  else if (b >= 2) cout <<(a>=1?"*":"")<<"yb^"<<b;
		  if (a == -1) cout <<"/ya";
		  hasWritten=true;
		}
		double Cba = getAnt(i)->getC(b,a);
		if (Cba != 0.0) {
		  if (linechange) cout<<endl<<" |                         ";
		  linechange=false;
		  cout<<" ";
		  if (Cba > 0.0 && hasWritten) cout<<"+ "; 
		  else if (Cba < 0.0) cout<<"- ";
		  iprc=2;
		  if ( int(abs(Cba) * 100 + 0.5) % 10 == 0) iprc = 1;
		  if (iprc == 1 && int(abs(Cba) * 10 +0.5) % 10 == 0) iprc=0;
		  if (a == -1 && b == 0) 
		    cout<<setprecision(iprc)<<abs(Cba);
		  else if (abs(Cba) != 1.0) 
		    cout<<setprecision(iprc)<<abs(Cba)<<"*";
		  if (a == 1) cout <<"yb";
		  else if (a >= 2) cout <<"yb^"<<a;
		  if (b == 1) cout <<(a>=1?"*":"")<<"ya";
		  else if (b >= 2) cout <<(a>=1?"*":"")<<"ya^"<<b;
		  if (a == -1) cout <<"/yb";

		  hasWritten=true;
		}
	      }
	    }
	    if (sectorOrdering) {
	      cout <<"\n |                        ";
	      for (int a=-1; a <= 3; a++) {
		double Da = getAnt(i)->getD(a);
		if (Da != 0.0) {
		  if (Da > 0.0) cout<<"+ ";
		  else cout<<"- ";
		  iprc=2;
		  if ( int(abs(Da) * 100 + 0.5) % 10 == 0) iprc = 1;
		  if (iprc == 1 && int(abs(Da) * 10 +0.5) % 10 == 0) iprc=0;
		  cout<<setprecision(iprc)<<abs(Da);
		  if (a == -1) cout <<"/(1-ya)/yb";
		  else if (a == 1) cout <<"*yb/(1-ya)";
		  else if (a >= 2) cout <<"*yb^"<<a<<"/(1-ya)";
		  cout<<" ";
		}
		double Ea = getAnt(i)->getE(a);
		if (Ea != 0.0) {
		  if (Ea > 0.0) cout<<"+ ";
		  else cout<<"- ";
		  iprc=2;
		  if ( int(abs(Ea) * 100 + 0.5) % 10 == 0) iprc = 1;
		  if (iprc == 1 && int(abs(Ea) * 10 +0.5) % 10 == 0) iprc=0;
		  cout<<setprecision(iprc)<<abs(Ea);
		  if (a == -1) cout <<"/(1-yb)/ya";
		  else if (a == 1) cout <<"*ya/(1-yb)";
		  else if (a >= 2) cout <<"*ya^"<<a<<"/(1-yb)";
		  cout<<" ";
		}
	      }
	    }
	    cout<<"\n |\n";
	  } else {
	    // Enter alternative input modes here
	  }
	} 

	// Switched off
	else {
	  cout<<" |   OFF \n | \n";	  
	}
      }
    }
    cout <<     " |-------------------------------------------"
	 << "------------------------------------------\n |\n";
    cout << " | References :"<<endl;
    cout << " |    VINCIA   : Giele, Kosower, Skands, PRD78(2008)014026"<<endl;
    cout << " |    PYTHIA8  : Sjostrand, Mrenna, Skands, CPC178(2008)852"<<endl;
    if (matching && matchingOrder >= 1 && isMGOn) {
      cout << " | The MADGRAPH interface relies on:"<<endl;
      cout << " |    HELAS    : Murayama et al., KEK-91-11 (1992)"<<endl;
      cout << " |    MADGRAPH : Alwall et al., JHEP09(2007)028"<<endl;
    }
  }
  cout << " |\n *-------  End VINCIA Global Initialization  "
       << "-----------------------------------------*\n\n";
  cout.setf(ios::right);
}  
  
// Print the list of dipoles.

void VinciaShower::list(ostream& os) {
  // Loop over dipole list and print it.
  for (int iDip = 0; iDip < int(branchelementals.size()); ++iDip) 
    if (branchelementals.size() == 1) branchelementals[iDip].list(os,true,true);
    else if ( iDip == 0 ) branchelementals[iDip].list(os,true,false);
    else if ( iDip == int(branchelementals.size())-1 ) branchelementals[iDip].list(os,false,true);
    else branchelementals[iDip].list(os);
}

void VinciaShower::printInfo(ostream& os) {    
  os << "\n";
  os << " *-------  VINCIA Statistics  -------------------------------------------------------------------------------------*\n";
  os << " |                                                                                                                 |\n"; 
  os << " | Matching was :                                              = ";
  if (matching && matchingOrder >= 1) {
    os <<setw(9)<< "on"<<setw(40)<<" "<<" |\n"; 
    os <<" | Matching:order                                              = "
       <<setw(9)<<matchingOrder<<setw(40)<<" "<<" |\n";
    os <<" | Matching:fullColor                                          = "
       <<setw(9)<<(matchingFullColor ? "on":"off")<<setw(40)<<" "<<" |\n";
    if (isMGOn) {
      os <<" | MGInterface                                                 = "
	 <<setw(9)<<(isMGOn ? "on":"off")<<setw(40)<<" "<<" |\n";
    }
  }
  else {
    cout<<setw(9)<<"off"<<setw(40)<<" "<<" |\n";
  } 
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n"; 
  os << " | Number of nonunity-weight events                            = "
     <<( (nNonunityWeight-nNegativeWeight <= TINY) ? "     none                    " 
	 : num2str(int(nNonunityWeight-nNegativeWeight),9)+" <-- WEIGHTED EVENTS" )<<setw(20)<<" "<<" |\n";
  os << " | Number of negative-weight events                            = "
     <<( (nNegativeWeight <= TINY) ? "     none" 
	 : num2str(int(nNegativeWeight),9)) <<setw(40)<<" "<<" |\n";
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n";   
  os << " |                                     weight(i)          Avg Wt   Avg Dev  rms(dev)     kUnwt     Expected effUnw |\n";
  os << " | This run                               i =     IsUnw     <w>     <w-1>                1/<w>   Max Wt  <w>/MaxWt |\n";
  os << " |"<<setw(4)<<" "<<"User settings                           0 "; 
  if (abs(1.-weightSumSave[0]/nTotWeights) < TINY) os << "   yes " ;
  else os << "    no ";
  os << num2str(weightSumSave[0]/nTotWeights,9)<<" ";
  os << num2str(weightSumSave[0]/nTotWeights-1.,9)<<" ";
  os << setw(9)<<"-"<<" ";
  os << num2str(nTotWeights/weightSumSave[0]);
      os << setw(9)<<"-"<<" ";
      os << setw(9)<<"-"<<" ";
      os <<" |\n";
  if (uncertaintyBands) {
    for (int iVar=1; iVar<nVariations(); iVar++) {
      if (iVar== 1) os<<" |"<<setw(4)<<" "<<"Var : VINCIA defaults                 ";
      if (iVar== 2) os<<" |"<<setw(4)<<" "<<"Var : AlphaStr-Max                    ";
      if (iVar== 3) os<<" |"<<setw(4)<<" "<<"Var : AlphaStr-Min                    ";
      if (iVar== 4) os<<" |"<<setw(4)<<" "<<"Var : Antennae-Max                    ";
      if (iVar== 5) os<<" |"<<setw(4)<<" "<<"Var : Antennae-Min                    ";
      if (iVar== 6) os<<" |"<<setw(4)<<" "<<"Var : RESERVED                        ";
      if (iVar== 7) os<<" |"<<setw(4)<<" "<<"Var : RESERVED                        ";
      if (iVar== 8) os<<" |"<<setw(4)<<" "<<"Var : Ordering-Stronger               ";
      if (iVar== 9) os<<" |"<<setw(4)<<" "<<"Var : Ordering-mDaughter              ";
      if (iVar==10) os<<" |"<<setw(4)<<" "<<"Var : ColorNLC-Max                    ";
      if (iVar==11) os<<" |"<<setw(4)<<" "<<"Var : ColorNLC-Min                    ";
      os << num2str(iVar,3)<<"   ";
      if (abs(1.-weightSumSave[iVar]/nTotWeights) < TINY) os << " yes " ;
      else os << "  no ";
      os << num2str(weightSumSave[iVar]/nTotWeights,9)<<" ";
      os << num2str(weightSumSave[iVar]/nTotWeights-1.,9)<<" ";
      os << setw(9)<<"-"<<" ";
      os << ((weightSumSave[iVar] != 0.) ? num2str(nTotWeights/weightSumSave[iVar],9): num2str(0.0,9));
      os << num2str(weightMaxSave[iVar],9)<<" ";
      os << ((weightMaxSave[iVar] != 0.) ? num2str(weightSumSave[iVar]/nTotWeights/weightMaxSave[iVar],9): num2str(0.0,9))<<" ";
      os <<" |\n";
    }
  }
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n"; 
  /*
  os << " | Largest Paccept (occurred at QE = "<<num2str(reweightMaxPSaveQ)
     <<" )               = "<<num2str(reweightMaxPSaveP)<<"   |\n";
  if (reweightMaxPSaveQ > TINY && reweightMaxQSaveQ > TINY) { 
    os << " | Largest QE with Paccept > 1 ( P = "<<num2str(reweightMaxQSaveP)
       <<" )               = "<<num2str(reweightMaxQSaveQ)<<"   |\n";
       }
  os << " |                                                                           |\n"; 
  */
  os << " *-------  End VINCIA Statistics ----------------------------------------------------------------------------------*\n\n";
}

//**********************************************************************

// Branchelemental class member functions

//*********

// Simple print utility, showing the contents of the Branchelemental

void Branchelemental::list(ostream& os, bool header, bool footer) { 
  if (header) {
    // Header.
    os << "\n --------  VINCIA Dipole-Antenna Listing  -----------------------------"
       << "---------------------------------------------------------- \n \n" 
       << "  sys   col      antenna-type   L aunts   parents  R uncles"
       << "         m      Qold    saved trial type   Qbranch   O"
       << "rd   nFail\n"
       << fixed << setprecision(3);
  }
  os << setw(5) << system <<setw(6) << old1.col() ;
  os << setw(9) << old1.name() << setw(9) << old2.name();
  os << setw(5) << iLL << setw(5) << iL;
  os << setw(5) << iOld1 << setw(5) << iOld2;
  os << setw(5) << iR << setw(5) << iRR;
  os << num2str(m(),10) << num2str(Qold,10); 
  if (hasTrial && Qnew > 0.0) {
    if (branchFlavor == 0) os << setw(20) << "gluon emission";
    else  os << setw(20) << "gluon splitting";
    os << num2str(Qnew,10);
    os << setw(6) << (unOrdering ? "n" : "y");
    os << num2str(nFailed,8);
  } else if (hasTrial) {
    os << num2str(0.0,30);
  } else {
    os << setw(30) << "-" <<setw(6) << (unOrdering ? "n" : "y");
    os << num2str(nFailed,8);
  }
  os << endl;   
  if (footer) {
  os << "\n --------  End VINCIA VinciaShower Dipole-Antenna Listing  ------------"
     << "----------------------------------------------------------\n";
  }
}

} // end namespace VINCIA
